@@ -2,15 +2,14 @@
 
 #include "StdAfx.h"
 
-#include "7zHandler.h"
-#include "7zFolderOutStream.h"
+#include "../../../Common/ComTry.h"
+
+#include "../../Common/ProgressUtils.h"
+
 #include "7zDecode.h"
 // #include "7z1Decode.h"
-
-#include "../../../Common/ComTry.h"
-#include "../../Common/StreamObjects.h"
-#include "../../Common/ProgressUtils.h"
-#include "../../Common/LimitedStreams.h"
+#include "7zFolderOutStream.h"
+#include "7zHandler.h"
 
 namespace NArchive {
 namespace N7z {
@@ -38,8 +37,8 @@ struct CExtractFolderInfo
   {
     if (fileIndex != kNumNoIndex)
     {
-      ExtractStatuses.Reserve(1);
-      ExtractStatuses.Add(true);
+      ExtractStatuses.ClearAndSetSize(1);
+      ExtractStatuses[0] = true;
     }
   };
 };
@@ -52,7 +51,7 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
   CMyComPtr<IArchiveExtractCallback> extractCallback = extractCallbackSpec;
   UInt64 importantTotalUnpacked = 0;
 
-  bool allFilesMode = (numItems == (UInt32)-1);
+  bool allFilesMode = (numItems == (UInt32)(Int32)-1);
   if (allFilesMode)
     numItems =
     #ifdef _7Z_VOL
@@ -68,18 +67,18 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
   if(_volumes.Size() != 1)
     return E_FAIL;
   const CVolume &volume = _volumes.Front();
-  const CArchiveDatabaseEx &_db = volume.Database;
+  const CDbEx &_db = volume.Database;
   IInStream *_inStream = volume.Stream;
   */
   
   CObjectVector<CExtractFolderInfo> extractFolderInfoVector;
-  for(UInt32 ii = 0; ii < numItems; ii++)
+  for (UInt32 ii = 0; ii < numItems; ii++)
   {
     // UInt32 fileIndex = allFilesMode ? indexIndex : indices[indexIndex];
     UInt32 ref2Index = allFilesMode ? ii : indices[ii];
     // const CRef2 &ref2 = _refs[ref2Index];
 
-    // for(UInt32 ri = 0; ri < ref2.Refs.Size(); ri++)
+    // for (UInt32 ri = 0; ri < ref2.Refs.Size(); ri++)
     {
       #ifdef _7Z_VOL
       // const CRef &ref = ref2.Refs[ri];
@@ -87,10 +86,10 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
 
       int volumeIndex = ref.VolumeIndex;
       const CVolume &volume = _volumes[volumeIndex];
-      const CArchiveDatabaseEx &db = volume.Database;
+      const CDbEx &db = volume.Database;
       UInt32 fileIndex = ref.ItemIndex;
       #else
-      const CArchiveDatabaseEx &db = _db;
+      const CDbEx &db = _db;
       UInt32 fileIndex = ref2Index;
       #endif
 
@@ -116,8 +115,7 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
             volumeIndex,
             #endif
             kNumNoIndex, folderIndex));
-        const CFolder &folderInfo = db.Folders[folderIndex];
-        UInt64 unpackSize = folderInfo.GetUnpackSize();
+        UInt64 unpackSize = db.GetFolderUnpackSize(folderIndex);
         importantTotalUnpacked += unpackSize;
         extractFolderInfoVector.Back().UnpackSize = unpackSize;
       }
@@ -138,7 +136,7 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
     }
   }
 
-  extractCallback->SetTotal(importantTotalUnpacked);
+  RINOK(extractCallback->SetTotal(importantTotalUnpacked));
 
   CDecoder decoder(
     #ifdef _ST_MODE
@@ -149,36 +147,35 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
     );
   // CDecoder1 decoder;
 
-  UInt64 currentTotalPacked = 0;
-  UInt64 currentTotalUnpacked = 0;
-  UInt64 totalFolderUnpacked;
-  UInt64 totalFolderPacked;
+  UInt64 totalPacked = 0;
+  UInt64 totalUnpacked = 0;
+  UInt64 curPacked, curUnpacked;
 
   CLocalProgress *lps = new CLocalProgress;
   CMyComPtr<ICompressProgressInfo> progress = lps;
   lps->Init(extractCallback, false);
 
-  for(int i = 0; i < extractFolderInfoVector.Size(); i++,
-      currentTotalUnpacked += totalFolderUnpacked,
-      currentTotalPacked += totalFolderPacked)
+  for (unsigned i = 0;; i++, totalUnpacked += curUnpacked, totalPacked += curPacked)
   {
-    lps->OutSize = currentTotalUnpacked;
-    lps->InSize = currentTotalPacked;
+    lps->OutSize = totalUnpacked;
+    lps->InSize = totalPacked;
     RINOK(lps->SetCur());
+
+    if (i >= extractFolderInfoVector.Size())
+      break;
     
     const CExtractFolderInfo &efi = extractFolderInfoVector[i];
-    totalFolderUnpacked = efi.UnpackSize;
-
-    totalFolderPacked = 0;
+    curUnpacked = efi.UnpackSize;
+    curPacked = 0;
 
     CFolderOutStream *folderOutStream = new CFolderOutStream;
     CMyComPtr<ISequentialOutStream> outStream(folderOutStream);
 
     #ifdef _7Z_VOL
     const CVolume &volume = _volumes[efi.VolumeIndex];
-    const CArchiveDatabaseEx &db = volume.Database;
+    const CDbEx &db = volume.Database;
     #else
-    const CArchiveDatabaseEx &db = _db;
+    const CDbEx &db = _db;
     #endif
 
     CNum startIndex;
@@ -187,7 +184,6 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
     else
       startIndex = db.FolderStartFileIndex[efi.FolderIndex];
 
-
     HRESULT result = folderOutStream->Init(&db,
         #ifdef _7Z_VOL
         volume.StartRef2Index,
@@ -203,12 +199,7 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
       continue;
 
     CNum folderIndex = efi.FolderIndex;
-    const CFolder &folderInfo = db.Folders[folderIndex];
-
-    totalFolderPacked = _db.GetFolderFullPackSize(folderIndex);
-
-    CNum packStreamIndex = db.FolderStartPackStreamIndex[folderIndex];
-    UInt64 folderStartPackPos = db.GetFolderStreamPos(folderIndex, 0);
+    curPacked = _db.GetFolderFullPackSize(folderIndex);
 
     #ifndef _NO_CRYPTO
     CMyComPtr<ICryptoGetTextPassword> getTextPassword;
@@ -219,26 +210,24 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
     try
     {
       #ifndef _NO_CRYPTO
-      bool passwordIsDefined;
+        bool isEncrypted = false;
+        bool passwordIsDefined = false;
       #endif
 
       HRESULT result = decoder.Decode(
           EXTERNAL_CODECS_VARS
           #ifdef _7Z_VOL
-          volume.Stream,
+            volume.Stream,
           #else
-          _inStream,
+            _inStream,
           #endif
-          folderStartPackPos,
-          &db.PackSizes[packStreamIndex],
-          folderInfo,
+          db.ArcInfo.DataStartPosition,
+          db, folderIndex,
           outStream,
           progress
-          #ifndef _NO_CRYPTO
-          , getTextPassword, passwordIsDefined
-          #endif
+          _7Z_DECODER_CRYPRO_VARS
           #if !defined(_7ZIP_ST) && !defined(_SFX)
-          , true, _numThreads
+            , true, _numThreads
           #endif
           );
 
@@ -249,7 +238,7 @@ STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
       }
       if (result == E_NOTIMPL)
       {
-        RINOK(folderOutStream->FlushCorrupted(NExtract::NOperationResult::kUnSupportedMethod));
+        RINOK(folderOutStream->FlushCorrupted(NExtract::NOperationResult::kUnsupportedMethod));
         continue;
       }
       if (result != S_OK)
