@@ -1,4 +1,4 @@
-// Tar/Update.h
+// TarUpdate.h
 
 #ifndef __TAR_UPDATE_H
 #define __TAR_UPDATE_H
@@ -11,20 +11,23 @@ namespace NTar {
 
 struct CUpdateItem
 {
-  bool NewData;
-  bool NewProperties;
   int IndexInArchive;
   int IndexInClient;
-
-  UInt32 Time;
+  Int64 MTime;
   UInt64 Size;
-  AString Name;
+  UInt32 Mode;
+  bool NewData;
+  bool NewProps;
   bool IsDir;
+  AString Name;
+  AString User;
+  AString Group;
 };
 
 HRESULT UpdateArchive(IInStream *inStream, ISequentialOutStream *outStream,
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
+    UINT codePage,
     IArchiveUpdateCallback *updateCallback);
 
 }}
