@@ -2,36 +2,50 @@
 
 #include "StdAfx.h"
 
-#include "Common/MyInitGuid.h"
-#include "Common/StringConvert.h"
-#include "Common/IntToString.h"
+#include <stdio.h>
 
-#include "Windows/PropVariant.h"
-#include "Windows/PropVariantConversions.h"
-#include "Windows/DLL.h"
-#include "Windows/FileDir.h"
-#include "Windows/FileName.h"
-#include "Windows/FileFind.h"
+#include "../../../Common/Defs.h"
+#include "../../../Common/MyInitGuid.h"
+
+#include "../../../Common/IntToString.h"
+#include "../../../Common/StringConvert.h"
+
+#include "../../../Windows/DLL.h"
+#include "../../../Windows/FileDir.h"
+#include "../../../Windows/FileFind.h"
+#include "../../../Windows/FileName.h"
+#include "../../../Windows/NtCheck.h"
+#include "../../../Windows/PropVariant.h"
+#include "../../../Windows/PropVariantConv.h"
 
 #include "../../Common/FileStreams.h"
+
 #include "../../Archive/IArchive.h"
+
 #include "../../IPassword.h"
-#include "../../MyVersion.h"
+#include "../../../../C/7zVersion.h"
 
+#ifdef _WIN32
+HINSTANCE g_hInstance = 0;
+#endif
 
+// Tou can find the list of all GUIDs in Guid.txt file.
+// use another CLSIDs, if you want to support other formats (zip, rar, ...).
 // {23170F69-40C1-278A-1000-000110070000}
-DEFINE_GUID(CLSID_CFormat7z, 
+DEFINE_GUID(CLSID_CFormat7z,
   0x23170F69, 0x40C1, 0x278A, 0x10, 0x00, 0x00, 0x01, 0x10, 0x07, 0x00, 0x00);
 
 using namespace NWindows;
+using namespace NFile;
+using namespace NDir;
 
 #define kDllName "7z.dll"
 
-static const char *kCopyrightString = MY_7ZIP_VERSION
-" ("  kDllName " client) "  
-MY_COPYRIGHT " " MY_DATE;
+static const char *kCopyrightString = "\n7-Zip " MY_VERSION
+" ("  kDllName " client) "
+MY_COPYRIGHT " " MY_DATE "\n";
 
-static const char *kHelpString = 
+static const char *kHelpString =
 "Usage: Client7z.exe [a | l | x ] archive.7z [fileName ...]\n"
 "Examples:\n"
 "  Client7z.exe a archive.7z f1.txt f2.txt  : compress two files to archive.7z\n"
@@ -39,51 +53,57 @@ static const char *kHelpString =
 "  Client7z.exe x archive.7z   : eXtract files from archive.7z\n";
 
 
-typedef UINT32 (WINAPI * CreateObjectFunc)(
-    const GUID *clsID, 
-    const GUID *interfaceID, 
-    void **outObject);
-
-#ifdef _WIN32
-#ifndef _UNICODE
-bool g_IsNT = false;
-static inline bool IsItWindowsNT()
+static AString FStringToConsoleString(const FString &s)
 {
-  OSVERSIONINFO versionInfo;
-  versionInfo.dwOSVersionInfoSize = sizeof(versionInfo);
-  if (!::GetVersionEx(&versionInfo)) 
-    return false;
-  return (versionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
+  return GetOemString(fs2us(s));
 }
-#endif
-#endif
 
-void PrintString(const UString &s)
+static FString CmdStringToFString(const char *s)
+{
+  return us2fs(GetUnicodeString(s));
+}
+
+static void PrintString(const UString &s)
 {
   printf("%s", (LPCSTR)GetOemString(s));
 }
 
-void PrintString(const AString &s)
+static void PrintString(const AString &s)
 {
   printf("%s", (LPCSTR)s);
 }
 
-void PrintNewLine()
+static void PrintNewLine()
 {
   PrintString("\n");
 }
 
-void PrintStringLn(const AString &s)
+static void PrintStringLn(const AString &s)
 {
   PrintString(s);
   PrintNewLine();
 }
 
+static void PrintError(const char *message, const FString &name)
+{
+  printf("Error: %s", (LPCSTR)message);
+  PrintNewLine();
+  PrintString(FStringToConsoleString(name));
+  PrintNewLine();
+}
+
+static void PrintError(const AString &s)
+{
+  PrintNewLine();
+  PrintString(s);
+  PrintNewLine();
+}
+
 static HRESULT IsArchiveItemProp(IInArchive *archive, UInt32 index, PROPID propID, bool &result)
 {
   NCOM::CPropVariant prop;
   RINOK(archive->GetProperty(index, propID, &prop));
-  if(prop.vt == VT_BOOL)
+  if (prop.vt == VT_BOOL)
     result = VARIANT_BOOLToBool(prop.boolVal);
   else if (prop.vt == VT_EMPTY)
     result = false;
@@ -94,7 +114,7 @@ static HRESULT IsArchiveItemProp(IInArchive *archive, UInt32 index, PROPID propI
 
 static HRESULT IsArchiveItemFolder(IInArchive *archive, UInt32 index, bool &result)
 {
-  return IsArchiveItemProp(archive, index, kpidIsFolder, result);
+  return IsArchiveItemProp(archive, index, kpidIsDir, result);
 }
 
 
@@ -105,7 +125,7 @@ static const wchar_t *kEmptyFileAlias = L"[Content]";
 // Archive Open callback class
 
 
-class CArchiveOpenCallback: 
+class CArchiveOpenCallback:
   public IArchiveOpenCallback,
   public ICryptoGetTextPassword,
   public CMyUnknownImp
@@ -139,22 +159,18 @@ STDMETHODIMP CArchiveOpenCallback::CryptoGetTextPassword(BSTR *password)
   if (!PasswordIsDefined)
   {
     // You can ask real password here from user
-    // Password = GetPassword(OutStream); 
+    // Password = GetPassword(OutStream);
     // PasswordIsDefined = true;
-    PrintStringLn("Password is not defined");
+    PrintError("Password is not defined");
     return E_ABORT;
   }
-  CMyComBSTR tempName(Password);
-  *password = tempName.Detach();
-  return S_OK;
+  return StringToBstr(Password, password);
 }
 
 
 //////////////////////////////////////////////////////////////
 // Archive Extracting callback class
 
-static const wchar_t *kCantDeleteOutputFile = L"ERROR: Can not delete output file ";
-
 static const char *kTestingString    =  "Testing     ";
 static const char *kExtractingString =  "Extracting  ";
 static const char *kSkippingString   =  "Skipping    ";
@@ -162,9 +178,13 @@ static const char *kSkippingString   =  "Skipping    ";
 static const char *kUnsupportedMethod = "Unsupported Method";
 static const char *kCRCFailed = "CRC Failed";
 static const char *kDataError = "Data Error";
-static const char *kUnknownError = "Unknown Error";
+static const char *kUnavailableData = "Unavailable data";
+static const char *kUnexpectedEnd = "Unexpected end of data";
+static const char *kDataAfterEnd = "There are some data after the end of the payload data";
+static const char *kIsNotArc = "Is not archive";
+static const char *kHeadersError = "Headers Error";
 
-class CArchiveExtractCallback: 
+class CArchiveExtractCallback:
   public IArchiveExtractCallback,
   public ICryptoGetTextPassword,
   public CMyUnknownImp
@@ -186,24 +206,24 @@ public:
 
 private:
   CMyComPtr<IInArchive> _archiveHandler;
-  UString _directoryPath;  // Output directory
+  FString _directoryPath;  // Output directory
   UString _filePath;       // name inside arcvhive
-  UString _diskFilePath;   // full path to file on disk
+  FString _diskFilePath;   // full path to file on disk
   bool _extractMode;
   struct CProcessedFileInfo
   {
-    FILETIME UTCLastWriteTime;
-    UInt32 Attributes;
-    bool IsDirectory;
-    bool AttributesAreDefined;
-    bool UTCLastWriteTimeIsDefined;
+    FILETIME MTime;
+    UInt32 Attrib;
+    bool isDir;
+    bool AttribDefined;
+    bool MTimeDefined;
   } _processedFileInfo;
 
   COutFileStream *_outFileStreamSpec;
   CMyComPtr<ISequentialOutStream> _outFileStream;
 
 public:
-  void Init(IInArchive *archiveHandler, const UString &directoryPath);
+  void Init(IInArchive *archiveHandler, const FString &directoryPath);
 
   UInt64 NumErrors;
   bool PasswordIsDefined;
@@ -212,12 +232,12 @@ public:
   CArchiveExtractCallback() : PasswordIsDefined(false) {}
 };
 
-void CArchiveExtractCallback::Init(IInArchive *archiveHandler, const UString &directoryPath)
+void CArchiveExtractCallback::Init(IInArchive *archiveHandler, const FString &directoryPath)
 {
   NumErrors = 0;
   _archiveHandler = archiveHandler;
   _directoryPath = directoryPath;
-  NFile::NName::NormalizeDirPathPrefix(_directoryPath);
+  NName::NormalizeDirPathPrefix(_directoryPath);
 }
 
 STDMETHODIMP CArchiveExtractCallback::SetTotal(UInt64 /* size */)
@@ -230,7 +250,7 @@ STDMETHODIMP CArchiveExtractCallback::SetCompleted(const UInt64 * /* completeVal
   return S_OK;
 }
 
-STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index, 
+STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index,
     ISequentialOutStream **outStream, Int32 askExtractMode)
 {
   *outStream = 0;
@@ -238,17 +258,17 @@ STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index,
 
   {
     // Get Name
-    NCOM::CPropVariant propVariant;
-    RINOK(_archiveHandler->GetProperty(index, kpidPath, &propVariant));
+    NCOM::CPropVariant prop;
+    RINOK(_archiveHandler->GetProperty(index, kpidPath, &prop));
     
     UString fullPath;
-    if(propVariant.vt == VT_EMPTY)
+    if (prop.vt == VT_EMPTY)
       fullPath = kEmptyFileAlias;
-    else 
+    else
     {
-      if(propVariant.vt != VT_BSTR)
+      if (prop.vt != VT_BSTR)
         return E_FAIL;
-      fullPath = propVariant.bstrVal;
+      fullPath = prop.bstrVal;
     }
     _filePath = fullPath;
   }
@@ -257,38 +277,38 @@ STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index,
     return S_OK;
 
   {
-    // Get Attributes
-    NCOM::CPropVariant propVariant;
-    RINOK(_archiveHandler->GetProperty(index, kpidAttributes, &propVariant));
-    if (propVariant.vt == VT_EMPTY)
+    // Get Attrib
+    NCOM::CPropVariant prop;
+    RINOK(_archiveHandler->GetProperty(index, kpidAttrib, &prop));
+    if (prop.vt == VT_EMPTY)
     {
-      _processedFileInfo.Attributes = 0;
-      _processedFileInfo.AttributesAreDefined = false;
+      _processedFileInfo.Attrib = 0;
+      _processedFileInfo.AttribDefined = false;
     }
     else
     {
-      if (propVariant.vt != VT_UI4)
-        throw "incorrect item";
-      _processedFileInfo.Attributes = propVariant.ulVal;
-      _processedFileInfo.AttributesAreDefined = true;
+      if (prop.vt != VT_UI4)
+        return E_FAIL;
+      _processedFileInfo.Attrib = prop.ulVal;
+      _processedFileInfo.AttribDefined = true;
     }
   }
 
-  RINOK(IsArchiveItemFolder(_archiveHandler, index, _processedFileInfo.IsDirectory));
+  RINOK(IsArchiveItemFolder(_archiveHandler, index, _processedFileInfo.isDir));
 
   {
     // Get Modified Time
-    NCOM::CPropVariant propVariant;
-    RINOK(_archiveHandler->GetProperty(index, kpidLastWriteTime, &propVariant));
-    _processedFileInfo.UTCLastWriteTimeIsDefined = false;
-    switch(propVariant.vt)
+    NCOM::CPropVariant prop;
+    RINOK(_archiveHandler->GetProperty(index, kpidMTime, &prop));
+    _processedFileInfo.MTimeDefined = false;
+    switch(prop.vt)
     {
       case VT_EMPTY:
-        // _processedFileInfo.UTCLastWriteTime = _utcLastWriteTimeDefault;
+        // _processedFileInfo.MTime = _utcMTimeDefault;
         break;
       case VT_FILETIME:
-        _processedFileInfo.UTCLastWriteTime = propVariant.filetime;
-        _processedFileInfo.UTCLastWriteTimeIsDefined = true;
+        _processedFileInfo.MTime = prop.filetime;
+        _processedFileInfo.MTimeDefined = true;
         break;
       default:
         return E_FAIL;
@@ -297,12 +317,10 @@ STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index,
   }
   {
     // Get Size
-    NCOM::CPropVariant propVariant;
-    RINOK(_archiveHandler->GetProperty(index, kpidSize, &propVariant));
-    bool newFileSizeDefined = (propVariant.vt != VT_EMPTY);
+    NCOM::CPropVariant prop;
+    RINOK(_archiveHandler->GetProperty(index, kpidSize, &prop));
     UInt64 newFileSize;
-    if (newFileSizeDefined)
-      newFileSize = ConvertPropVariantToUInt64(propVariant);
+    /* bool newFileSizeDefined = */ ConvertPropVariantToUInt64(prop, newFileSize);
   }
 
   
@@ -310,24 +328,24 @@ STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index,
     // Create folders for file
     int slashPos = _filePath.ReverseFind(WCHAR_PATH_SEPARATOR);
     if (slashPos >= 0)
-      NFile::NDirectory::CreateComplexDirectory(_directoryPath + _filePath.Left(slashPos));
+      CreateComplexDir(_directoryPath + us2fs(_filePath.Left(slashPos)));
   }
 
-  UString fullProcessedPath = _directoryPath + _filePath;
+  FString fullProcessedPath = _directoryPath + us2fs(_filePath);
   _diskFilePath = fullProcessedPath;
 
-  if (_processedFileInfo.IsDirectory)
+  if (_processedFileInfo.isDir)
   {
-    NFile::NDirectory::CreateComplexDirectory(fullProcessedPath);
+    CreateComplexDir(fullProcessedPath);
   }
   else
   {
-    NFile::NFind::CFileInfoW fileInfo;
-    if(NFile::NFind::FindFile(fullProcessedPath, fileInfo))
+    NFind::CFileInfo fi;
+    if (fi.Find(fullProcessedPath))
     {
-      if (!NFile::NDirectory::DeleteFileAlways(fullProcessedPath))
+      if (!DeleteFileAlways(fullProcessedPath))
       {
-        PrintString(UString(kCantDeleteOutputFile) + fullProcessedPath);
+        PrintError("Can not delete output file", fullProcessedPath);
         return E_ABORT;
       }
     }
@@ -336,7 +354,7 @@ STDMETHODIMP CArchiveExtractCallback::GetStream(UInt32 index,
     CMyComPtr<ISequentialOutStream> outStreamLoc(_outFileStreamSpec);
     if (!_outFileStreamSpec->Open(fullProcessedPath, CREATE_ALWAYS))
     {
-      PrintString((UString)L"can not open output file " + fullProcessedPath);
+      PrintError("Can not open output file", fullProcessedPath);
       return E_ABORT;
     }
     _outFileStream = outStreamLoc;
@@ -350,20 +368,13 @@ STDMETHODIMP CArchiveExtractCallback::PrepareOperation(Int32 askExtractMode)
   _extractMode = false;
   switch (askExtractMode)
   {
-    case NArchive::NExtract::NAskMode::kExtract:
-      _extractMode = true;
+    case NArchive::NExtract::NAskMode::kExtract:  _extractMode = true; break;
   };
   switch (askExtractMode)
   {
-    case NArchive::NExtract::NAskMode::kExtract:
-      PrintString(kExtractingString);
-      break;
-    case NArchive::NExtract::NAskMode::kTest:
-      PrintString(kTestingString);
-      break;
-    case NArchive::NExtract::NAskMode::kSkip:
-      PrintString(kSkippingString);
-      break;
+    case NArchive::NExtract::NAskMode::kExtract:  PrintString(kExtractingString); break;
+    case NArchive::NExtract::NAskMode::kTest:  PrintString(kTestingString); break;
+    case NArchive::NExtract::NAskMode::kSkip:  PrintString(kSkippingString); break;
   };
   PrintString(_filePath);
   return S_OK;
@@ -371,36 +382,66 @@ STDMETHODIMP CArchiveExtractCallback::PrepareOperation(Int32 askExtractMode)
 
 STDMETHODIMP CArchiveExtractCallback::SetOperationResult(Int32 operationResult)
 {
-  switch(operationResult)
+  switch (operationResult)
   {
     case NArchive::NExtract::NOperationResult::kOK:
       break;
     default:
     {
       NumErrors++;
-      PrintString("     ");
-      switch(operationResult)
+      PrintString("  :  ");
+      const char *s = NULL;
+      switch (operationResult)
       {
-        case NArchive::NExtract::NOperationResult::kUnSupportedMethod:
-          PrintString(kUnsupportedMethod);
+        case NArchive::NExtract::NOperationResult::kUnsupportedMethod:
+          s = kUnsupportedMethod;
           break;
         case NArchive::NExtract::NOperationResult::kCRCError:
-          PrintString(kCRCFailed);
+          s = kCRCFailed;
           break;
         case NArchive::NExtract::NOperationResult::kDataError:
-          PrintString(kDataError);
+          s = kDataError;
           break;
-        default:
-          PrintString(kUnknownError);
+        case NArchive::NExtract::NOperationResult::kUnavailable:
+          s = kUnavailableData;
+          break;
+        case NArchive::NExtract::NOperationResult::kUnexpectedEnd:
+          s = kUnexpectedEnd;
+          break;
+        case NArchive::NExtract::NOperationResult::kDataAfterEnd:
+          s = kDataAfterEnd;
+          break;
+        case NArchive::NExtract::NOperationResult::kIsNotArc:
+          s = kIsNotArc;
+          break;
+        case NArchive::NExtract::NOperationResult::kHeadersError:
+          s = kHeadersError;
+          break;
+      }
+      if (s)
+      {
+        PrintString("Error : ");
+        PrintString(s);
+      }
+      else
+      {
+        char temp[16];
+        ConvertUInt32ToString(operationResult, temp);
+        PrintString("Error #");
+        PrintString(temp);
       }
     }
   }
 
-  if(_outFileStream != NULL && _processedFileInfo.UTCLastWriteTimeIsDefined)
-    _outFileStreamSpec->SetLastWriteTime(&_processedFileInfo.UTCLastWriteTime);
+  if (_outFileStream != NULL)
+  {
+    if (_processedFileInfo.MTimeDefined)
+      _outFileStreamSpec->SetMTime(&_processedFileInfo.MTime);
+    RINOK(_outFileStreamSpec->Close());
+  }
   _outFileStream.Release();
-  if (_extractMode && _processedFileInfo.AttributesAreDefined)
-    NFile::NDirectory::MySetFileAttributes(_diskFilePath, _processedFileInfo.Attributes);
+  if (_extractMode && _processedFileInfo.AttribDefined)
+    SetFileAttrib(_diskFilePath, _processedFileInfo.Attrib);
   PrintNewLine();
   return S_OK;
 }
@@ -411,14 +452,12 @@ STDMETHODIMP CArchiveExtractCallback::CryptoGetTextPassword(BSTR *password)
   if (!PasswordIsDefined)
   {
     // You can ask real password here from user
-    // Password = GetPassword(OutStream); 
+    // Password = GetPassword(OutStream);
     // PasswordIsDefined = true;
-    PrintStringLn("Password is not defined");
+    PrintError("Password is not defined");
     return E_ABORT;
   }
-  CMyComBSTR tempName(Password);
-  *password = tempName.Detach();
-  return S_OK;
+  return StringToBstr(Password, password);
 }
 
 
@@ -427,18 +466,19 @@ STDMETHODIMP CArchiveExtractCallback::CryptoGetTextPassword(BSTR *password)
 // Archive Creating callback class
 
 struct CDirItem
-{ 
-  UInt32 Attributes;
-  FILETIME CreationTime;
-  FILETIME LastAccessTime;
-  FILETIME LastWriteTime;
+{
   UInt64 Size;
+  FILETIME CTime;
+  FILETIME ATime;
+  FILETIME MTime;
   UString Name;
-  UString FullPath;
-  bool IsDirectory() const { return (Attributes & FILE_ATTRIBUTE_DIRECTORY) != 0 ; }
+  FString FullPath;
+  UInt32 Attrib;
+
+  bool isDir() const { return (Attrib & FILE_ATTRIBUTE_DIRECTORY) != 0 ; }
 };
 
-class CArchiveUpdateCallback: 
+class CArchiveUpdateCallback:
   public IArchiveUpdateCallback2,
   public ICryptoGetTextPassword2,
   public CMyUnknownImp
@@ -451,8 +491,8 @@ public:
   STDMETHOD(SetCompleted)(const UInt64 *completeValue);
 
   // IUpdateCallback2
-  STDMETHOD(EnumProperties)(IEnumSTATPROPSTG **enumerator);  
-  STDMETHOD(GetUpdateItemInfo)(UInt32 index, 
+  STDMETHOD(EnumProperties)(IEnumSTATPROPSTG **enumerator);
+  STDMETHOD(GetUpdateItemInfo)(UInt32 index,
       Int32 *newData, Int32 *newProperties, UInt32 *indexInArchive);
   STDMETHOD(GetProperty)(UInt32 index, PROPID propID, PROPVARIANT *value);
   STDMETHOD(GetStream)(UInt32 index, ISequentialInStream **inStream);
@@ -467,7 +507,7 @@ public:
   UString VolName;
   UString VolExt;
 
-  UString DirPrefix;
+  FString DirPrefix;
   const CObjectVector<CDirItem> *DirItems;
 
   bool PasswordIsDefined;
@@ -476,7 +516,7 @@ public:
 
   bool m_NeedBeClosed;
 
-  UStringVector FailedFiles;
+  FStringVector FailedFiles;
   CRecordVector<HRESULT> FailedCodes;
 
   CArchiveUpdateCallback(): PasswordIsDefined(false), AskPassword(false), DirItems(0) {};
@@ -509,26 +549,26 @@ STDMETHODIMP CArchiveUpdateCallback::EnumProperties(IEnumSTATPROPSTG ** /* enume
   return E_NOTIMPL;
 }
 
-STDMETHODIMP CArchiveUpdateCallback::GetUpdateItemInfo(UInt32 /* index */, 
+STDMETHODIMP CArchiveUpdateCallback::GetUpdateItemInfo(UInt32 /* index */,
       Int32 *newData, Int32 *newProperties, UInt32 *indexInArchive)
 {
-  if(newData != NULL)
+  if (newData != NULL)
     *newData = BoolToInt(true);
-  if(newProperties != NULL)
+  if (newProperties != NULL)
     *newProperties = BoolToInt(true);
-  if(indexInArchive != NULL)
-    *indexInArchive = UInt32(-1);
+  if (indexInArchive != NULL)
+    *indexInArchive = (UInt32)(Int32)-1;
   return S_OK;
 }
 
 STDMETHODIMP CArchiveUpdateCallback::GetProperty(UInt32 index, PROPID propID, PROPVARIANT *value)
 {
-  NWindows::NCOM::CPropVariant propVariant;
+  NCOM::CPropVariant prop;
   
   if (propID == kpidIsAnti)
   {
-    propVariant = false;
-    propVariant.Detach(value);
+    prop = false;
+    prop.Detach(value);
     return S_OK;
   }
 
@@ -536,30 +576,16 @@ STDMETHODIMP CArchiveUpdateCallback::GetProperty(UInt32 index, PROPID propID, PR
     const CDirItem &dirItem = (*DirItems)[index];
     switch(propID)
     {
-      case kpidPath:
-        propVariant = dirItem.Name;
-        break;
-      case kpidIsFolder:
-        propVariant = dirItem.IsDirectory();
-        break;
-      case kpidSize:
-        propVariant = dirItem.Size;
-        break;
-      case kpidAttributes:
-        propVariant = dirItem.Attributes;
-        break;
-      case kpidLastAccessTime:
-        propVariant = dirItem.LastAccessTime;
-        break;
-      case kpidCreationTime:
-        propVariant = dirItem.CreationTime;
-        break;
-      case kpidLastWriteTime:
-        propVariant = dirItem.LastWriteTime;
-        break;
+      case kpidPath:  prop = dirItem.Name; break;
+      case kpidIsDir:  prop = dirItem.isDir(); break;
+      case kpidSize:  prop = dirItem.Size; break;
+      case kpidAttrib:  prop = dirItem.Attrib; break;
+      case kpidCTime:  prop = dirItem.CTime; break;
+      case kpidATime:  prop = dirItem.ATime; break;
+      case kpidMTime:  prop = dirItem.MTime; break;
     }
   }
-  propVariant.Detach(value);
+  prop.Detach(value);
   return S_OK;
 }
 
@@ -588,14 +614,14 @@ STDMETHODIMP CArchiveUpdateCallback::GetStream(UInt32 index, ISequentialInStream
   const CDirItem &dirItem = (*DirItems)[index];
   GetStream2(dirItem.Name);
  
-  if(dirItem.IsDirectory())
+  if (dirItem.isDir())
     return S_OK;
 
   {
     CInFileStream *inStreamSpec = new CInFileStream;
     CMyComPtr<ISequentialInStream> inStreamLoc(inStreamSpec);
-    UString path = DirPrefix + dirItem.FullPath;
-    if(!inStreamSpec->Open(path))
+    FString path = DirPrefix + dirItem.FullPath;
+    if (!inStreamSpec->Open(path))
     {
       DWORD sysError = ::GetLastError();
       FailedCodes.Add(sysError);
@@ -603,7 +629,7 @@ STDMETHODIMP CArchiveUpdateCallback::GetStream(UInt32 index, ISequentialInStream
       // if (systemError == ERROR_SHARING_VIOLATION)
       {
         PrintNewLine();
-        PrintStringLn("WARNING: can't open file");
+        PrintError("WARNING: can't open file");
         // PrintString(NError::MyFormatMessageW(systemError));
         return S_FALSE;
       }
@@ -632,10 +658,10 @@ STDMETHODIMP CArchiveUpdateCallback::GetVolumeSize(UInt32 index, UInt64 *size)
 
 STDMETHODIMP CArchiveUpdateCallback::GetVolumeStream(UInt32 index, ISequentialOutStream **volumeStream)
 {
-  wchar_t temp[32];
-  ConvertUInt64ToString(index + 1, temp);
+  wchar_t temp[16];
+  ConvertUInt32ToString(index + 1, temp);
   UString res = temp;
-  while (res.Length() < 2)
+  while (res.Len() < 2)
     res = UString(L'0') + res;
   UString fileName = VolName;
   fileName += L'.';
@@ -643,7 +669,7 @@ STDMETHODIMP CArchiveUpdateCallback::GetVolumeStream(UInt32 index, ISequentialOu
   fileName += VolExt;
   COutFileStream *streamSpec = new COutFileStream;
   CMyComPtr<ISequentialOutStream> streamLoc(streamSpec);
-  if(!streamSpec->Create(fileName, false))
+  if (!streamSpec->Create(us2fs(fileName), false))
     return ::GetLastError();
   *volumeStream = streamLoc.Detach();
   return S_OK;
@@ -651,105 +677,106 @@ STDMETHODIMP CArchiveUpdateCallback::GetVolumeStream(UInt32 index, ISequentialOu
 
 STDMETHODIMP CArchiveUpdateCallback::CryptoGetTextPassword2(Int32 *passwordIsDefined, BSTR *password)
 {
-  if (!PasswordIsDefined) 
+  if (!PasswordIsDefined)
   {
     if (AskPassword)
     {
       // You can ask real password here from user
-      // Password = GetPassword(OutStream); 
+      // Password = GetPassword(OutStream);
       // PasswordIsDefined = true;
-      PrintStringLn("Password is not defined");
+      PrintError("Password is not defined");
       return E_ABORT;
     }
   }
   *passwordIsDefined = BoolToInt(PasswordIsDefined);
-  CMyComBSTR tempName(Password);
-  *password = tempName.Detach();
-  return S_OK;
+  return StringToBstr(Password, password);
 }
 
-
-
 //////////////////////////////////////////////////////////////////////////
 // Main function
 
-int 
-#ifdef _MSC_VER
-__cdecl 
-#endif
-main(int argc, char* argv[])
+#define NT_CHECK_FAIL_ACTION PrintError("Unsupported Windows version"); return 1;
+
+int MY_CDECL main(int numArgs, const char *args[])
 {
-  #ifdef _WIN32
-  #ifndef _UNICODE
-  g_IsNT = IsItWindowsNT();
-  #endif
-  #endif
+  NT_CHECK
 
   PrintStringLn(kCopyrightString);
 
-  if (argc < 3)
+  if (numArgs < 3)
   {
     PrintStringLn(kHelpString);
     return 1;
   }
-  NWindows::NDLL::CLibrary library;
-  if (!library.Load(TEXT(kDllName)))
+  NDLL::CLibrary lib;
+  if (!lib.Load(NDLL::GetModuleDirPrefix() + FTEXT(kDllName)))
   {
-    PrintStringLn("Can not load library");
+    PrintError("Can not load 7-zip library");
     return 1;
   }
-  CreateObjectFunc createObjectFunc = (CreateObjectFunc)library.GetProcAddress("CreateObject");
-  if (createObjectFunc == 0)
+  Func_CreateObject createObjectFunc = (Func_CreateObject)lib.GetProc("CreateObject");
+  if (!createObjectFunc)
   {
-    PrintStringLn("Can not get CreateObject");
+    PrintError("Can not get CreateObject");
     return 1;
   }
 
-  AString command = argv[1];
-  UString archiveName = GetUnicodeString(argv[2], CP_OEMCP);
-  if (command.CompareNoCase("a") == 0)
+  char c;
+  {
+    AString command = args[1];
+    if (command.Len() != 1)
+    {
+      PrintError("incorrect command");
+      return 1;
+    }
+    c = (char)MyCharLower_Ascii(command[0]);
+  }
+  FString archiveName = CmdStringToFString(args[2]);
+  if (c == 'a')
   {
     // create archive command
-    if (argc < 4)
+    if (numArgs < 4)
     {
       PrintStringLn(kHelpString);
       return 1;
     }
     CObjectVector<CDirItem> dirItems;
-    int i;
-    for (i = 3; i < argc; i++)
     {
-      CDirItem item;
-      UString name = GetUnicodeString(argv[i], CP_OEMCP);
-      
-      NFile::NFind::CFileInfoW fileInfo;
-      if (!NFile::NFind::FindFile(name, fileInfo))
+      int i;
+      for (i = 3; i < numArgs; i++)
       {
-        PrintString(UString(L"Can't find file") + name);
-        return 1;
+        CDirItem di;
+        FString name = CmdStringToFString(args[i]);
+        
+        NFind::CFileInfo fi;
+        if (!fi.Find(name))
+        {
+          PrintError("Can't find file", name);
+          return 1;
+        }
+        
+        di.Attrib = fi.Attrib;
+        di.Size = fi.Size;
+        di.CTime = fi.CTime;
+        di.ATime = fi.ATime;
+        di.MTime = fi.MTime;
+        di.Name = fs2us(name);
+        di.FullPath = name;
+        dirItems.Add(di);
       }
-
-      item.Attributes = fileInfo.Attributes;
-      item.Size = fileInfo.Size;
-      item.CreationTime = fileInfo.CreationTime;
-      item.LastAccessTime = fileInfo.LastAccessTime;
-      item.LastWriteTime = fileInfo.LastWriteTime;
-      item.Name = name;
-      item.FullPath = name;
-      dirItems.Add(item);
     }
     COutFileStream *outFileStreamSpec = new COutFileStream;
     CMyComPtr<IOutStream> outFileStream = outFileStreamSpec;
     if (!outFileStreamSpec->Create(archiveName, false))
     {
-      PrintStringLn("can't create archive file");
+      PrintError("can't create archive file");
       return 1;
     }
 
     CMyComPtr<IOutArchive> outArchive;
     if (createObjectFunc(&CLSID_CFormat7z, &IID_IOutArchive, (void **)&outArchive) != S_OK)
     {
-      PrintStringLn("Can not get class object");
+      PrintError("Can not get class object");
       return 1;
     }
 
@@ -759,44 +786,68 @@ main(int argc, char* argv[])
     // updateCallbackSpec->PasswordIsDefined = true;
     // updateCallbackSpec->Password = L"1";
 
+    /*
+    {
+      const wchar_t *names[] =
+      {
+        L"s",
+        L"x"
+      };
+      const unsigned kNumProps = ARRAY_SIZE(names);
+      NCOM::CPropVariant values[kNumProps] =
+      {
+        false,    // solid mode OFF
+        (UInt32)9 // compression level = 9 - ultra
+      };
+      CMyComPtr<ISetProperties> setProperties;
+      outArchive->QueryInterface(IID_ISetProperties, (void **)&setProperties);
+      if (!setProperties)
+      {
+        PrintError("ISetProperties unsupported");
+        return 1;
+      }
+      RINOK(setProperties->SetProperties(names, values, kNumProps));
+    }
+    */
+    
     HRESULT result = outArchive->UpdateItems(outFileStream, dirItems.Size(), updateCallback);
     updateCallbackSpec->Finilize();
     if (result != S_OK)
     {
-      PrintStringLn("Update Error");
+      PrintError("Update Error");
       return 1;
     }
-    for (i = 0; i < updateCallbackSpec->FailedFiles.Size(); i++)
+    FOR_VECTOR (i, updateCallbackSpec->FailedFiles)
     {
       PrintNewLine();
-      PrintString((UString)L"Error for file: " + updateCallbackSpec->FailedFiles[i]);
+      PrintError("Error for file", updateCallbackSpec->FailedFiles[i]);
     }
     if (updateCallbackSpec->FailedFiles.Size() != 0)
       return 1;
   }
   else
   {
-    if (argc != 3)
+    if (numArgs != 3)
     {
       PrintStringLn(kHelpString);
       return 1;
     }
 
     bool listCommand;
-    if (command.CompareNoCase("l") == 0)
+    if (c == 'l')
       listCommand = true;
-    else if (command.CompareNoCase("x") == 0)
+    else if (c == 'x')
       listCommand = false;
     else
     {
-      PrintStringLn("incorrect command");
+      PrintError("incorrect command");
       return 1;
     }
   
     CMyComPtr<IInArchive> archive;
     if (createObjectFunc(&CLSID_CFormat7z, &IID_IInArchive, (void **)&archive) != S_OK)
     {
-      PrintStringLn("Can not get class object");
+      PrintError("Can not get class object");
       return 1;
     }
     
@@ -805,7 +856,7 @@ main(int argc, char* argv[])
     
     if (!fileSpec->Open(archiveName))
     {
-      PrintStringLn("Can not open archive file");
+      PrintError("Can not open archive file", archiveName);
       return 1;
     }
 
@@ -818,7 +869,7 @@ main(int argc, char* argv[])
       
       if (archive->Open(file, 0, openCallback) != S_OK)
       {
-        PrintStringLn("Can not open archive");
+        PrintError("Can not open file as archive", archiveName);
         return 1;
       }
     }
@@ -827,25 +878,28 @@ main(int argc, char* argv[])
     {
       // List command
       UInt32 numItems = 0;
-      archive->GetNumberOfItems(&numItems);  
+      archive->GetNumberOfItems(&numItems);
       for (UInt32 i = 0; i < numItems; i++)
       {
         {
           // Get uncompressed size of file
-          NWindows::NCOM::CPropVariant propVariant;
-          archive->GetProperty(i, kpidSize, &propVariant);
-          UString s = ConvertPropVariantToString(propVariant);
+          NCOM::CPropVariant prop;
+          archive->GetProperty(i, kpidSize, &prop);
+          char s[32];
+          ConvertPropVariantToShortString(prop, s);
           PrintString(s);
           PrintString("  ");
         }
         {
           // Get name of file
-          NWindows::NCOM::CPropVariant propVariant;
-          archive->GetProperty(i, kpidPath, &propVariant);
-          UString s = ConvertPropVariantToString(propVariant);
-          PrintString(s);
+          NCOM::CPropVariant prop;
+          archive->GetProperty(i, kpidPath, &prop);
+          if (prop.vt == VT_BSTR)
+            PrintString(prop.bstrVal);
+          else if (prop.vt != VT_EMPTY)
+            PrintString("ERROR!");
         }
-        PrintString("\n");
+        PrintNewLine();
       }
     }
     else
@@ -853,11 +907,16 @@ main(int argc, char* argv[])
       // Extract command
       CArchiveExtractCallback *extractCallbackSpec = new CArchiveExtractCallback;
       CMyComPtr<IArchiveExtractCallback> extractCallback(extractCallbackSpec);
-      extractCallbackSpec->Init(archive, L""); // second parameter is output folder path
+      extractCallbackSpec->Init(archive, FTEXT("")); // second parameter is output folder path
       extractCallbackSpec->PasswordIsDefined = false;
       // extractCallbackSpec->PasswordIsDefined = true;
       // extractCallbackSpec->Password = L"1";
-      archive->Extract(NULL, (UInt32)(Int32)(-1), false, extractCallback);
+      HRESULT result = archive->Extract(NULL, (UInt32)(Int32)(-1), false, extractCallback);
+      if (result != S_OK)
+      {
+        PrintError("Extract Error");
+        return 1;
+      }
     }
   }
   return 0;
