@@ -175,7 +175,7 @@ protected:
 
 		int wmId = event.GetId();
 
-		if (wmId >= kToolbarStartID)
+		if (wmId >= kMenuCmdID_Toolbar_Start && wmId < kMenuCmdID_Toolbar_End)
 		{
 			ExecuteCommand(wmId);
 			return ; // 0;
@@ -365,35 +365,35 @@ void MyFrame::PopulateToolbar(wxToolBar* p_toolBar)
 	toolBar.SetToolBitmapSize(wxSize(kWidth,kHeight));
 	
 	
-	
+#if 0 // FIXME
     msg = LangString(0x03020400); // { kAddCommand, IDB_ADD, IDB_ADD2, IDS_ADD, 0x03020400}
-    if (msg == L"") msg = L"Add";
+    if (msg == ,L"") msg = ,L"Add";
 	toolBar.AddTool(kAddCommand, (const wchar_t *)msg, wxGetBitmapFromMemory(ADD2));
 
     msg = LangString(0x03020401); // { kExtractCommand, IDB_EXTRACT, IDB_EXTRACT2, IDS_EXTRACT, 0x03020401}
-    if (msg == L"") msg = L"Extract";	
+    if (msg == ,L"") msg = ,L"Extract";	
 	toolBar.AddTool(kExtractCommand,(const wchar_t *)msg, wxGetBitmapFromMemory(EXTRACT2));
 	
     msg = LangString(0x03020402); // { kTestCommand , IDB_TEST, IDB_TEST2, IDS_TEST, 0x03020402}
-    if (msg == L"") msg = L"Test";	
+    if (msg == ,L"") msg = ,L"Test";	
 	toolBar.AddTool(kTestCommand,(const wchar_t *)msg, wxGetBitmapFromMemory(TEST2));
 
 	toolBar.AddSeparator();
 
     msg = LangString(0x03020420); // { IDM_COPY_TO, IDB_COPY, IDB_COPY2, IDS_BUTTON_COPY, 0x03020420}
-    if (msg == L"") msg = L"Copy";		
+    if (msg == ,L"") msg = ,L"Copy";		
 	toolBar.AddTool(IDM_COPY_TO, (const wchar_t *)msg, wxGetBitmapFromMemory(COPY2));
 	
     msg = LangString(0x03020421); // { IDM_MOVE_TO, IDB_MOVE, IDB_MOVE2, IDS_BUTTON_MOVE, 0x03020421}
-    if (msg == L"") msg = L"Move";		
+    if (msg == ,L"") msg = ,L"Move";		
 	toolBar.AddTool(IDM_MOVE_TO, (const wchar_t *)msg, wxGetBitmapFromMemory(MOVE2));
 	
     msg = LangString(0x03020422); // { IDM_DELETE, IDB_DELETE, IDB_DELETE2, IDS_BUTTON_DELETE, 0x03020422}
-    if (msg == L"") msg = L"Delete";	
+    if (msg == ,L"") msg = ,L"Delete";	
 	toolBar.AddTool(IDM_DELETE, (const wchar_t *)msg, wxGetBitmapFromMemory(DELETE2));
 	
     msg = LangString(0x03020423); // { IDM_FILE_PROPERTIES, IDB_INFO, IDB_INFO2, IDS_BUTTON_INFO, 0x03020423}
-    if (msg == L"") msg = L"Info";	
+    if (msg == ,L"") msg = ,L"Info";	
 	toolBar.AddTool(IDM_FILE_PROPERTIES, (const wchar_t *)msg, wxGetBitmapFromMemory(INFO2));
 
 	////////////////////////////////////////////////////////
@@ -425,6 +425,7 @@ void MyFrame::PopulateToolbar(wxToolBar* p_toolBar)
 
 	///////////////////////////////////////////////////////
 
+#endif
 	toolBar.Realize();
 
 	// toolBar->SetRows(!(toolBar->IsVertical()) ? m_rows : 10 / m_rows);
@@ -481,23 +482,32 @@ void rc_MyLoadMenu(HWND hWnd)
 	wxMenu *m;
 	wxMenu *m_file = m = new wxMenu;
 	{
-		m->Append(IDM_FILE_OPEN, _T("&Open"));  // FIXME "&Open\tEnter" - don't use Enter to support combobox enter ...
-		m->Append(IDM_FILE_OPEN_INSIDE,_T("Open &Inside\tCtrl+PgDn"));
-		m->Append(IDM_FILE_OPEN_OUTSIDE,_T("Open O&utside\tShift+Enter"));
-//		m->Append(IDM_FILE_EDIT,_T("&Edit\tF4"));
+		m->Append(IDM_OPEN, _T("&Open"));  // FIXME "&Open\tEnter" - don't use Enter to support combobox enter ...
+		m->Append(IDM_OPEN_INSIDE,_T("Open &Inside\tCtrl+PgDn"));
+		m->Append(IDM_OPEN_OUTSIDE,_T("Open O&utside\tShift+Enter"));
+		m->Append(IDM_FILE_VIEW,_T("&View\tF3"));
+		m->Append(IDM_FILE_EDIT,_T("&Edit\tF4"));
 		m->AppendSeparator();
-//		m->Append(IDM_RENAME,_T("Rena&me\tF2"));
+		m->Append(IDM_RENAME,_T("Rena&me\tF2"));
 		m->Append(IDM_COPY_TO,_T("&Copy To...\tF5"));
 		m->Append(IDM_MOVE_TO,_T("&Move To...\tF6"));
 		m->Append(IDM_DELETE,_T("&Delete\tDel"));
 		m->AppendSeparator();
-		m->Append(IDM_FILE_SPLIT,_T("&Split file..."));
-		m->Append(IDM_FILE_COMBINE,_T("Com&bine files..."));
+		m->Append(IDM_SPLIT,_T("&Split file..."));
+		m->Append(IDM_COMBINE,_T("Com&bine files..."));
 		m->AppendSeparator();
-//		m->Append(IDM_FILE_PROPERTIES,_T("P&roperties\tAlt+Enter"));
-//		m->Append(IDM_FILE_COMMENT,_T("Comme&nt\tCtrl+Z"));
-		m->Append(IDM_FILE_CRC,_T("Calculate checksum"));
-		m->Append(IDM_FILE_DIFF,_T("Di&ff"));
+		m->Append(IDM_PROPERTIES,_T("P&roperties\tAlt+Enter"));
+		m->Append(IDM_COMMENT,_T("Comme&nt\tCtrl+Z"));
+
+		wxMenu * subMenuCRC = new wxMenu;
+		subMenuCRC->Append(IDM_CRC32   ,_T("CRC-32"));
+		subMenuCRC->Append(IDM_CRC64   ,_T("CRC-64"));
+		subMenuCRC->Append(IDM_SHA1    ,_T("SHA-1"));
+		subMenuCRC->Append(IDM_SHA256  ,_T("SHA-256"));
+		subMenuCRC->Append(IDM_HASH_ALL,_T("*"));
+		m->AppendSubMenu(subMenuCRC,_T("CRC"));
+
+		m->Append(IDM_DIFF,_T("Di&ff"));
 		m->AppendSeparator();
 		m->Append(IDM_CREATE_FOLDER,_T("Create Folder\tF7"));
 		m->Append(IDM_CREATE_FILE,_T("Create File\tCtrl+N"));
@@ -506,11 +516,10 @@ void rc_MyLoadMenu(HWND hWnd)
 	}
 	wxMenu *m_edit = m = new wxMenu;
 	{
-		m->Append(IDM_EDIT_CUT, _T("Cu&t\tCtrl+X"))->Enable(true);              // GRAYED
-		// m->Enable(IDM_EDIT_CUT, false);
-		m->Append(IDM_EDIT_COPY, _T("&Copy\tCtrl+C"))->Enable(true);            // GRAYED
-		m->Append(IDM_EDIT_PASTE, _T("&Paste\tCtrl+V"))->Enable(true);          // GRAYED
-		m->AppendSeparator();
+		// m->Append(IDM_EDIT_CUT, _T("Cu&t\tCtrl+X"))->Enable(true);              // GRAYED
+		// m->Append(IDM_EDIT_COPY, _T("&Copy\tCtrl+C"))->Enable(true);            // GRAYED
+		// m->Append(IDM_EDIT_PASTE, _T("&Paste\tCtrl+V"))->Enable(true);          // GRAYED
+		// m->AppendSeparator();
 		m->Append(IDM_SELECT_ALL, _T("Select &All\tShift+[Grey +]")); 
 		m->Append(IDM_DESELECT_ALL, _T("Deselect All\tShift+[Grey -]")); 
 		m->Append(IDM_INVERT_SELECTION, _T("&Invert Selection\tGrey *"));   
@@ -547,7 +556,9 @@ void rc_MyLoadMenu(HWND hWnd)
 		}
 		m->AppendSeparator();
 */
-		m->Append(IDM_OPEN_ROOT_FOLDER, _T("Open Root Folder\t" STRING_PATH_SEPARATOR));        
+		// NO "/" is used on Unix Path ... m->Append(IDM_OPEN_ROOT_FOLDER, _T("Open Root Folder\t" STRING_PATH_SEPARATOR));        
+		m->Append(IDM_OPEN_ROOT_FOLDER, _T("Open Root Folder\t" "\\"));        
+
 		m->Append(IDM_OPEN_PARENT_FOLDER, _T("Up One Level\tBackspace"));
 		m->Append(IDM_FOLDERS_HISTORY, _T("Folders History...\tAlt+F12"));
 		m->AppendSeparator();
@@ -559,7 +570,7 @@ void rc_MyLoadMenu(HWND hWnd)
 			wxMenu* subMenu = new wxMenu;
 			for (int i = 0; i < 10; i++)
 			{
-				UString s = LangString(IDS_BOOKMARK, 0x03000720);
+				UString s = LangString(IDS_BOOKMARK);
 				s += L" ";
 				wchar_t c = (wchar_t)(L'0' + i);
 				s += c;
@@ -573,19 +584,18 @@ void rc_MyLoadMenu(HWND hWnd)
 		m->AppendSeparator();
 		for (int i = 0; i < 10; i++)
 		{
-			UString path = g_App.AppState.FastFolders.GetString(i);
-			const int kMaxSize = 100;
-			const int kFirstPartSize = kMaxSize / 2;
-			if (path.Length() > kMaxSize)
-			{
-				path = path.Left(kFirstPartSize) + UString(L" ... ") +
-					path.Right(kMaxSize - kFirstPartSize);
-			}
-			UString s = path;
-			if (s.IsEmpty())
-				s = L"-";
-			s += L"\tAlt+";
-			s += (wchar_t)(L'0' + i);
+      UString s = g_App.AppState.FastFolders.GetString(i);
+      const int kMaxSize = 100;
+      const int kFirstPartSize = kMaxSize / 2;
+      if (s.Len() > kMaxSize)
+      {
+        s.Delete(kFirstPartSize, s.Len() - kMaxSize);
+        s.Insert(kFirstPartSize, L" ... ");
+      }
+      if (s.IsEmpty())
+        s = L'-';
+      s += L"\tAlt+";
+      s += (wchar_t)(L'0' + i);
 			// menu.AppendItem(MF_STRING, kOpenBookmarkMenuID + i, s);
 			m->Append( kOpenBookmarkMenuID + i, wxString(s));
 		}
@@ -621,149 +631,182 @@ static CStringTable g_stringTable[] =
 {
   /* resource.rc */	  
   /***************/
-	{ IDS_APP_TITLE, L"7-Zip File Manager" },
+  { IDS_BOOKMARK   ,L"Bookmark" },
 
-	{ IDS_COPY                , L"Copy" },
-	{ IDS_MOVE                , L"Move" },
-	{ IDS_COPY_TO             , L"Copy to:" },
-	{ IDS_MOVE_TO             , L"Move to:" },
-	{ IDS_COPYING             , L"Copying..." },
-	{ IDS_MOVING              , L"Moving..." },
-	{ IDS_CANNOT_COPY         , L"You cannot move or copy items for such folders." },
-	{ IDS_SPLITTING           , L"Splitting..." },
-	{ IDS_SPLIT_CONFIRM_TITLE , L"Confirm Splitting" },
-	{ IDS_SPLIT_CONFIRM_MESSAGE , L"Are you sure you want to split file into {0} volumes?" },
-	{ IDS_SPLIT_VOL_MUST_BE_SMALLER , L"Volume size must be smaller than size of original file" },
+  { IDS_OPTIONS    ,L"Options" },
 
-	{ IDS_COMBINE             , L"Combine Files" },
-	{ IDS_COMBINE_TO          , L"&Combine to:" },
-	{ IDS_COMBINING           , L"Combining..." },
-	{ IDS_COMBINE_SELECT_ONE_FILE , L"Select only first file" },
+  { IDS_N_SELECTED_ITEMS  ,L"{0} object(s) selected" },
 
-	{ IDS_CHECKSUM_CALCULATING , L"Checksum calculating..." },
-	{ IDS_CHECKSUM_INFORMATION , L"Checksum information" },
-	{ IDS_CHECKSUM_CRC_DATA     , L"CRC checksum for data:" },
-	{ IDS_CHECKSUM_CRC_DATA_NAMES , L"CRC checksum for data and names:" },
+  { IDS_FILE_EXIST  ,L"File {0} is already exist" },
+  { IDS_WANT_UPDATE_MODIFIED_FILE  ,L"File '{0}' was modified.\nDo you want to update it in the archive?" },
+  { IDS_CANNOT_UPDATE_FILE   ,L"Can not update file\n'{0}'" },
+  { IDS_CANNOT_START_EDITOR  ,L"Cannot start editor." },
+  { IDS_VIRUS                ,L"The file looks like a virus (the file name contains long spaces in name)." },
+  { IDS_MESSAGE_UNSUPPORTED_OPERATION_FOR_LONG_PATH_FOLDER ,L"The operation cannot be called from a folder that has a long path." },
+  { IDS_SELECT_ONE_FILE      ,L"You must select one file" },
+  { IDS_SELECT_FILES         ,L"You must select one or more files" },
+  { IDS_TOO_MANY_ITEMS       ,L"Too many items" },
 
-	{ IDS_SCANNING , L"Scanning..." },
+  { IDS_COPY      ,L"Copy" },
+  { IDS_MOVE      ,L"Move" },
+  { IDS_COPY_TO   ,L"Copy to:" },
+  { IDS_MOVE_TO   ,L"Move to:" },
+  { IDS_COPYING   ,L"Copying..." },
+  { IDS_MOVING    ,L"Moving..." },
+  { IDS_RENAMING  ,L"Renaming..." },
 
-	{ IDS_PROPERTIES , L"Properties" },
+  { IDS_OPERATION_IS_NOT_SUPPORTED  ,L"Operation is not supported." },
+  { IDS_ERROR_RENAMING  ,L"Error Renaming File or Folder" },
+  { IDS_CONFIRM_FILE_COPY   ,L"Confirm File Copy" },
+  { IDS_WANT_TO_COPY_FILES  ,L"Are you sure you want to copy files to archive" },
 
-	{ IDS_OPERATION_IS_NOT_SUPPORTED , L"Operation is not supported." },
+  { IDS_CONFIRM_FILE_DELETE   ,L"Confirm File Delete" },
+  { IDS_CONFIRM_FOLDER_DELETE ,L"Confirm Folder Delete" },
+  { IDS_CONFIRM_ITEMS_DELETE  ,L"Confirm Multiple File Delete" },
+  { IDS_WANT_TO_DELETE_FILE   ,L"Are you sure you want to delete '{0}'?" },
+  { IDS_WANT_TO_DELETE_FOLDER ,L"Are you sure you want to delete the folder '{0}' and all its contents?" },
+  { IDS_WANT_TO_DELETE_ITEMS  ,L"Are you sure you want to delete these {0} items?" },
+  { IDS_DELETING              ,L"Deleting..." },
+  { IDS_ERROR_DELETING        ,L"Error Deleting File or Folder" },
+  { IDS_ERROR_LONG_PATH_TO_RECYCLE  ,L"The system cannot move a file with long path to the Recycle Bin" },
   
-	{ IDS_CONFIRM_FILE_DELETE , L"Confirm File Delete" },
-	{ IDS_CONFIRM_FOLDER_DELETE , L"Confirm Folder Delete" },
-	{ IDS_CONFIRM_ITEMS_DELETE , L"Confirm Multiple File Delete" },
-	{ IDS_WANT_TO_DELETE_FILE , L"Are you sure you want to delete '{0}'?" },
-	{ IDS_WANT_TO_DELETE_FOLDER , L"Are you sure you want to delete the folder '{0}' and all its contents?" },
-	{ IDS_WANT_TO_DELETE_ITEMS  , L"Are you sure you want to delete these {0} items?" },
-	{ IDS_DELETING            , L"Deleting..." },
-	{ IDS_ERROR_DELETING      , L"Error Deleting File or Folder" },
-	{ IDS_RENAMING            , L"Renaming..." },
-	{ IDS_ERROR_RENAMING      , L"Error Renaming File or Folder" },
-	{ IDS_CONFIRM_FILE_COPY   , L"Confirm File Copy" },
-	{ IDS_WANT_TO_COPY_FILES  , L"Are you sure you want to copy files to archive" },
-  
-	{ IDS_CREATE_FOLDER       , L"Create Folder" },
-	{ IDS_CREATE_FOLDER_NAME  , L"Folder name:" },
-	{ IDS_CREATE_FOLDER_DEFAULT_NAME , L"New Folder" },
-	{ IDS_CREATE_FOLDER_ERROR , L"Error Creating Folder" },
-	{ IDS_CREATE_FILE         , L"Create File" },
-	{ IDS_CREATE_FILE_NAME    , L"File Name:" },
-	{ IDS_CREATE_FILE_DEFAULT_NAME , L"New File" },
-	{ IDS_CREATE_FILE_ERROR   , L"Error Creating File" },
-	{ IDS_SELECT              , L"Select" },
-	{ IDS_DESELECT            , L"Deselect" },
-	{ IDS_SELECT_MASK         , L"Mask:" },
-	{ IDS_FOLDERS_HISTORY     , L"Folders History" },
-	{ IDS_N_SELECTED_ITEMS    , L"{0} object(s) selected" },
-	{ IDS_FILES_COLON         , L"Files:" },
-	{ IDS_FOLDERS_COLON       , L"Folders:" },
-	{ IDS_SIZE_COLON          , L"Size:" },
+  { IDS_CREATE_FOLDER       ,L"Create Folder" },
+  { IDS_CREATE_FILE         ,L"Create File" },
+  { IDS_CREATE_FOLDER_NAME  ,L"Folder name:" },
+  { IDS_CREATE_FILE_NAME    ,L"File Name:" },
+  { IDS_CREATE_FOLDER_DEFAULT_NAME  ,L"New Folder" },
+  { IDS_CREATE_FILE_DEFAULT_NAME    ,L"New File" },
+  { IDS_CREATE_FOLDER_ERROR ,L"Error Creating Folder" },
+  { IDS_CREATE_FILE_ERROR   ,L"Error Creating File" },
+
+  { IDS_COMMENT      ,L"Comment" },
+  { IDS_COMMENT2     ,L"&Comment:" },
+  { IDS_SELECT       ,L"Select" },
+  { IDS_DESELECT     ,L"Deselect" },
+  { IDS_SELECT_MASK  ,L"Mask:" },
+
+  { IDS_PROPERTIES   ,L"Properties" },
+  { IDS_FOLDERS_HISTORY  ,L"Folders History" },
+
+  { IDS_COMPUTER   ,L"Computer" },
+  { IDS_NETWORK    ,L"Network" },
+  { IDS_DOCUMENTS  ,L"Documents" },
+  { IDS_SYSTEM     ,L"System" },
+
+  { IDS_ADD            ,L"Add" },
+  { IDS_EXTRACT        ,L"Extract" },
+  { IDS_TEST           ,L"Test" },
+  { IDS_BUTTON_COPY    ,L"Copy" },
+  { IDS_BUTTON_MOVE    ,L"Move" },
+  { IDS_BUTTON_DELETE  ,L"Delete" },
+  { IDS_BUTTON_INFO    ,L"Info" },
+
+  { IDS_SPLITTING            ,L"Splitting..." },
+  { IDS_SPLIT_CONFIRM_TITLE  ,L"Confirm Splitting" },
+  { IDS_SPLIT_CONFIRM_MESSAGE  ,L"Are you sure you want to split file into {0} volumes?" },
+  { IDS_SPLIT_VOL_MUST_BE_SMALLER ,L"Volume size must be smaller than size of original file" },
+
+  { IDS_COMBINE     ,L"Combine Files" },
+  { IDS_COMBINE_TO  ,L"&Combine to:" },
+  { IDS_COMBINING   ,L"Combining..." },
+  { IDS_COMBINE_SELECT_ONE_FILE  ,L"Select only first part of split file" },
+  { IDS_COMBINE_CANT_DETECT_SPLIT_FILE  ,L"Can not detect file as split file" },
+  { IDS_COMBINE_CANT_FIND_MORE_THAN_ONE_PART  ,L"Can not find more than one part of split file" },
 
-	{ IDS_PROP_TOTAL_SIZE     , L"Total Size" },
-	{ IDS_PROP_FREE_SPACE     , L"Free Space" },
-	{ IDS_PROP_CLUSTER_SIZE   , L"Cluster Size" },
-	{ IDS_PROP_VOLUME_NAME    , L"Label" },
-	{ IDS_PROP_LOCAL_NAME     , L"Local Name" },
-	{ IDS_PROP_PROVIDER       , L"Provider" },
-	{ IDS_OPTIONS             , L"Options" },
-	{ IDS_COMMENT             , L"Comment" },
-	{ IDS_COMMENT2            , L"&Comment:" },
-	{ IDS_SYSTEM              , L"System" },
-	{ IDS_TOO_MANY_ITEMS      , L"Too many items" },
-	{ IDS_WANT_UPDATE_MODIFIED_FILE , L"File '{0}' was modified.\nDo you want to update it in the archive?" },
-	{ IDS_CANNOT_UPDATE_FILE  , L"Can not update file\n'{0}'" },
-	{ IDS_CANNOT_START_EDITOR , L"Cannot start editor." },
-	{ IDS_OPENNING            , L"Opening..." },
-	{ IDS_ADD                 , L"Add" },
-	{ IDS_EXTRACT             , L"Extract" },
-	{ IDS_TEST                , L"Test" },
-	{ IDS_BUTTON_COPY         , L"Copy" },
-	{ IDS_BUTTON_MOVE         , L"Move" },
-	{ IDS_BUTTON_DELETE       , L"Delete" },
-	{ IDS_BUTTON_INFO         , L"Info" },
-	{ IDS_BOOKMARK            , L"Bookmark" },
-	{ IDS_COMPUTER            , L"Computer" },
-	{ IDS_NETWORK             , L"Network" },
 
-	{ IDS_PROGRESS_TESTING    , L"Testing" },
-	{ IDS_MESSAGE_NO_ERRORS   , L"There are no errors" },
 
   /* PropertyName.rc */	  
   /*******************/
-	{ IDS_PROP_PATH       , L"Path" },
-	{ IDS_PROP_NAME       , L"Name" },
-	{ IDS_PROP_EXTENSION  , L"Extension" },
-	{ IDS_PROP_IS_FOLDER  , L"Folder" },
-	{ IDS_PROP_SIZE       , L"Size" },
-	{ IDS_PROP_PACKED_SIZE , L"Packed Size" },
-	{ IDS_PROP_ATTRIBUTES , L"Attributes" },
-	{ IDS_PROP_CTIME , L"Created" },
-	{ IDS_PROP_ATIME , L"Accessed" },
-	{ IDS_PROP_MTIME , L"Modified" },
-	{ IDS_PROP_SOLID      , L"Solid" },
-	{ IDS_PROP_C0MMENTED  , L"Commented" },
-	{ IDS_PROP_ENCRYPTED  , L"Encrypted" },
-	{ IDS_PROP_DICTIONARY_SIZE , L"Dictionary Size" },
-	{ IDS_PROP_SPLIT_BEFORE , L"Split Before" },
-	{ IDS_PROP_SPLIT_AFTER , L"Split After" },
-	{ IDS_PROP_CRC        , L"CRC" },
-	{ IDS_PROP_FILE_TYPE  , L"Type" },
-	{ IDS_PROP_ANTI       , L"Anti" },
-	{ IDS_PROP_METHOD     , L"Method" },
-	{ IDS_PROP_HOST_OS    , L"Host OS" },
-	{ IDS_PROP_FILE_SYSTEM , L"File System" },
-	{ IDS_PROP_USER , L"User" },
-	{ IDS_PROP_GROUP , L"Group" },
-	{ IDS_PROP_BLOCK , L"Block" },
-	{ IDS_PROP_COMMENT , L"Comment" },
-	{ IDS_PROP_POSITION , L"Position" },
-	{ IDS_PROP_PREFIX , L"Path Prefix" },
-	{ IDS_PROP_FOLDERS , L"Folders" },
-	{ IDS_PROP_FILES , L"Files" },
-	{ IDS_PROP_VERSION , L"Version" },
-	{ IDS_PROP_VOLUME , L"Volume" },
-	{ IDS_PROP_IS_VOLUME , L"Multivolume" },
-	{ IDS_PROP_OFFSET , L"Offset" },
-	{ IDS_PROP_LINKS , L"Links" },
-	{ IDS_PROP_NUM_BLOCKS , L"Blocks" },
-	{ IDS_PROP_NUM_VOLUMES , L"Volumes" },
+  { IDS_PROP_PATH         ,L"Path" },
+  { IDS_PROP_NAME         ,L"Name" },
+  { IDS_PROP_EXTENSION    ,L"Extension" },
+  { IDS_PROP_IS_FOLDER    ,L"Folder" },
+  { IDS_PROP_SIZE         ,L"Size" },
+  { IDS_PROP_PACKED_SIZE  ,L"Packed Size" },
+  { IDS_PROP_ATTRIBUTES   ,L"Attributes" },
+  { IDS_PROP_CTIME        ,L"Created" },
+  { IDS_PROP_ATIME        ,L"Accessed" },
+  { IDS_PROP_MTIME        ,L"Modified" },
+  { IDS_PROP_SOLID        ,L"Solid" },
+  { IDS_PROP_C0MMENTED    ,L"Commented" },
+  { IDS_PROP_ENCRYPTED    ,L"Encrypted" },
+  { IDS_PROP_SPLIT_BEFORE ,L"Split Before" },
+  { IDS_PROP_SPLIT_AFTER  ,L"Split After" },
+  { IDS_PROP_DICTIONARY_SIZE ,L"Dictionary" },
+  { IDS_PROP_CRC          ,L"CRC" },
+  { IDS_PROP_FILE_TYPE    ,L"Type" },
+  { IDS_PROP_ANTI         ,L"Anti" },
+  { IDS_PROP_METHOD       ,L"Method" },
+  { IDS_PROP_HOST_OS      ,L"Host OS" },
+  { IDS_PROP_FILE_SYSTEM  ,L"File System" },
+  { IDS_PROP_USER         ,L"User" },
+  { IDS_PROP_GROUP        ,L"Group" },
+  { IDS_PROP_BLOCK        ,L"Block" },
+  { IDS_PROP_COMMENT      ,L"Comment" },
+  { IDS_PROP_POSITION     ,L"Position" },
+  { IDS_PROP_PREFIX       ,L"Path Prefix" },
+  { IDS_PROP_FOLDERS      ,L"Folders" },
+  { IDS_PROP_FILES        ,L"Files" },
+  { IDS_PROP_VERSION      ,L"Version" },
+  { IDS_PROP_VOLUME       ,L"Volume" },
+  { IDS_PROP_IS_VOLUME    ,L"Multivolume" },
+  { IDS_PROP_OFFSET       ,L"Offset" },
+  { IDS_PROP_LINKS        ,L"Links" },
+  { IDS_PROP_NUM_BLOCKS   ,L"Blocks" },
+  { IDS_PROP_NUM_VOLUMES  ,L"Volumes" },
 
-	{ IDS_PROP_BIT64 , L"64-bit" },
-	{ IDS_PROP_BIG_ENDIAN , L"Big-endian" },
-	{ IDS_PROP_CPU , L"CPU" },
-	{ IDS_PROP_PHY_SIZE , L"Physical Size" },
-	{ IDS_PROP_HEADERS_SIZE , L"Headers Size" },
-	{ IDS_PROP_CHECKSUM , L"Checksum" },
-	{ IDS_PROP_CHARACTS , L"Characteristics" },
-	{ IDS_PROP_VA , L"Virtual Address" },
-	{ IDS_PROP_ID , L"ID" },
-	{ IDS_PROP_SHORT_NAME , L"Short Name" },
-	{ IDS_PROP_CREATOR_APP , L"Creator Application" },
-	{ IDS_PROP_SECTOR_SIZE , L"Sector Size" },
-	{ IDS_PROP_POSIX_ATTRIB , L"Mode" },
-	{ IDS_PROP_LINK , L"Link" },
+  { IDS_PROP_BIT64        ,L"64-bit" },
+  { IDS_PROP_BIG_ENDIAN   ,L"Big-endian" },
+  { IDS_PROP_CPU          ,L"CPU" },
+  { IDS_PROP_PHY_SIZE     ,L"Physical Size" },
+  { IDS_PROP_HEADERS_SIZE ,L"Headers Size" },
+  { IDS_PROP_CHECKSUM     ,L"Checksum" },
+  { IDS_PROP_CHARACTS     ,L"Characteristics" },
+  { IDS_PROP_VA           ,L"Virtual Address" },
+  { IDS_PROP_ID           ,L"ID" },
+  { IDS_PROP_SHORT_NAME   ,L"Short Name" },
+  { IDS_PROP_CREATOR_APP  ,L"Creator Application" },
+  { IDS_PROP_SECTOR_SIZE  ,L"Sector Size" },
+  { IDS_PROP_POSIX_ATTRIB ,L"Mode" },
+  { IDS_PROP_SYM_LINK     ,L"Symbolic Link" },
+  { IDS_PROP_ERROR        ,L"Error" },
+  { IDS_PROP_TOTAL_SIZE   ,L"Total Size" },
+  { IDS_PROP_FREE_SPACE   ,L"Free Space" },
+  { IDS_PROP_CLUSTER_SIZE ,L"Cluster Size" },
+  { IDS_PROP_VOLUME_NAME  ,L"Label" },
+  { IDS_PROP_LOCAL_NAME   ,L"Local Name" },
+  { IDS_PROP_PROVIDER     ,L"Provider" },
+  { IDS_PROP_NT_SECURITY  ,L"NT Security" },
+  { IDS_PROP_ALT_STREAM   ,L"Alternate Stream" },
+  { IDS_PROP_AUX          ,L"Aux" },
+  { IDS_PROP_DELETED      ,L"Deleted" },
+  { IDS_PROP_IS_TREE      ,L"Is Tree" },
+  { IDS_PROP_SHA1         ,L"SHA-1" },
+  { IDS_PROP_SHA256       ,L"SHA-256" },
+  { IDS_PROP_ERROR_TYPE   ,L"Error Type" },
+  { IDS_PROP_NUM_ERRORS   ,L"Errors" },
+  { IDS_PROP_ERROR_FLAGS  ,L"Errors" },
+  { IDS_PROP_WARNING_FLAGS ,L"Warnings" },
+  { IDS_PROP_WARNING      ,L"Warning" },
+  { IDS_PROP_NUM_STREAMS  ,L"Streams" },
+  { IDS_PROP_NUM_ALT_STREAMS ,L"Alternate Streams" },
+  { IDS_PROP_ALT_STREAMS_SIZE ,L"Alternate Streams Size" },
+  { IDS_PROP_VIRTUAL_SIZE ,L"Virtual Size" },
+  { IDS_PROP_UNPACK_SIZE  ,L"Unpack Size" },
+  { IDS_PROP_TOTAL_PHY_SIZE ,L"Total Physical Size" },
+  { IDS_PROP_VOLUME_INDEX ,L"Volume Index" },
+  { IDS_PROP_SUBTYPE      ,L"SubType" },
+  { IDS_PROP_SHORT_COMMENT ,L"Short Comment" },
+  { IDS_PROP_CODE_PAGE    ,L"Code Page" },
+  { IDS_PROP_IS_NOT_ARC_TYPE  ,L"Is not archive type" },
+  { IDS_PROP_PHY_SIZE_CANT_BE_DETECTED ,L"Physical Size can't be detected" },
+  { IDS_PROP_ZEROS_TAIL_IS_ALLOWED ,L"Zeros Tail Is Allowed" },
+  { IDS_PROP_TAIL_SIZE ,L"Tail Size" },
+  { IDS_PROP_EMB_STUB_SIZE ,L"Embedded Stub Size" },
+  { IDS_PROP_NT_REPARSE   ,L"Link" },
+  { IDS_PROP_HARD_LINK    ,L"Hard Link" },
+  { IDS_PROP_INODE        ,L"iNode" },
+  { IDS_PROP_STREAM_ID    ,L"Stream ID" },
  
 	{ 0 , 0 }
 };
