@@ -4,22 +4,22 @@
 
 #include "resource.h"
 
-#include "Windows/Thread.h"
+#include "../../../Windows/FileName.h"
+#include "../../../Windows/Thread.h"
 
 #include "../Agent/Agent.h"
 
 #include "LangUtils.h"
 #include "OpenCallback.h"
 #include "PluginLoader.h"
-#include "RegistryAssociations.h"
 #include "RegistryPlugins.h"
 
 using namespace NWindows;
-using namespace NRegistryAssociations;
 
 struct CThreadArchiveOpen
 {
   UString Path;
+  UString ArcFormat;
   CMyComPtr<IInStream> InStream;
   CMyComPtr<IFolderManager> FolderManager;
   CMyComPtr<IProgress> OpenCallback;
@@ -30,9 +30,12 @@ struct CThreadArchiveOpen
 
   void Process()
   {
-    OpenCallbackSpec->ProgressDialog.WaitCreating();
-    Result = FolderManager->OpenFolderFile(InStream, Path, &Folder, OpenCallback);
-    OpenCallbackSpec->ProgressDialog.MyClose();
+    try
+    {
+      CProgressCloser closer(OpenCallbackSpec->ProgressDialog);
+      Result = FolderManager->OpenFolderFile(InStream, Path, ArcFormat, &Folder, OpenCallback);
+    }
+    catch(...) { Result = E_FAIL; }
   }
   
   static THREAD_FUNC_DECL MyThreadFunction(void *param)
@@ -42,18 +45,40 @@ struct CThreadArchiveOpen
   }
 };
 
-static int FindPlugin(const CObjectVector<CPluginInfo> &plugins,
-    const UString &pluginName)
+/*
+static int FindPlugin(const CObjectVector<CPluginInfo> &plugins, const UString &pluginName)
 {
   for (int i = 0; i < plugins.Size(); i++)
     if (plugins[i].Name.CompareNoCase(pluginName) == 0)
       return i;
   return -1;
 }
+*/
+
+static const FChar kExtensionDelimiter = FTEXT('.');
+
+static void SplitNameToPureNameAndExtension(const FString &fullName,
+    FString &pureName, FString &extensionDelimiter, FString &extension)
+{
+  int index = fullName.ReverseFind(kExtensionDelimiter);
+  if (index < 0)
+  {
+    pureName = fullName;
+    extensionDelimiter.Empty();
+    extension.Empty();
+  }
+  else
+  {
+    pureName.SetFrom(fullName, index);
+    extensionDelimiter = kExtensionDelimiter;
+    extension = fullName.Ptr(index + 1);
+  }
+}
 
 HRESULT OpenFileFolderPlugin(
     IInStream *inStream,
-    const UString &path,
+    const FString &path,
+    const UString &arcFormat,
     HMODULE *module,
     IFolderFolder **resultFolder,
     HWND parentWindow,
@@ -64,22 +89,22 @@ HRESULT OpenFileFolderPlugin(
   ReadFileFolderPluginInfoList(plugins);
 #endif
 
-  UString extension, name, pureName, dot;
+  FString extension, name, pureName, dot;
 
-  int slashPos = path.ReverseFind(WCHAR_PATH_SEPARATOR);
-  UString dirPrefix;
-  UString fileName;
+  int slashPos = path.ReverseFind(FCHAR_PATH_SEPARATOR);
+  FString dirPrefix;
+  FString fileName;
   if (slashPos >= 0)
   {
-    dirPrefix = path.Left(slashPos + 1);
-    fileName = path.Mid(slashPos + 1);
+    dirPrefix.SetFrom(path, slashPos + 1);
+    fileName = path.Ptr(slashPos + 1);
   }
   else
     fileName = path;
 
-  NFile::NName::SplitNameToPureNameAndExtension(fileName, pureName, dot, extension);
+  SplitNameToPureNameAndExtension(fileName, pureName, dot, extension);
 
-#ifdef _WIN32
+  /*
   if (!extension.IsEmpty())
   {
     CExtInfo extInfo;
@@ -97,13 +122,15 @@ HRESULT OpenFileFolderPlugin(
       }
     }
   }
+  */
 
-  for (int i = 0; i < plugins.Size(); i++)
+#ifdef _WIN32
+  FOR_VECTOR (i, plugins)
   {
     const CPluginInfo &plugin = plugins[i];
     if (!plugin.ClassIDDefined)
       continue;
-#endif // #ifdef _WIN32
+#endif
     CPluginLibrary library;
 
     CThreadArchiveOpen t;
@@ -124,31 +151,34 @@ HRESULT OpenFileFolderPlugin(
     t.OpenCallbackSpec->ParentWindow = parentWindow;
 
     if (inStream)
-      t.OpenCallbackSpec->SetSubArchiveName(fileName);
+      t.OpenCallbackSpec->SetSubArchiveName(fs2us(fileName));
     else
       t.OpenCallbackSpec->LoadFileInfo(dirPrefix, fileName);
 
     t.InStream = inStream;
-    t.Path = path;
+    t.Path = fs2us(path);
+    t.ArcFormat = arcFormat;
 
-    UString progressTitle = LangString(IDS_OPENNING, 0x03020283);
+    UString progressTitle = LangString(IDS_OPENNING);
     t.OpenCallbackSpec->ProgressDialog.MainWindow = parentWindow;
-    t.OpenCallbackSpec->ProgressDialog.MainTitle = LangString(IDS_APP_TITLE, 0x03000000);
-    t.OpenCallbackSpec->ProgressDialog.MainAddTitle = progressTitle + UString(L" ");
+    t.OpenCallbackSpec->ProgressDialog.MainTitle = L"7-Zip"; // LangString(IDS_APP_TITLE);
+    t.OpenCallbackSpec->ProgressDialog.MainAddTitle = progressTitle + L' ';
+    // FIXME t.OpenCallbackSpec->ProgressDialog.WaitMode = true;
 
-    NWindows::CThread thread;
-    if (thread.Create(CThreadArchiveOpen::MyThreadFunction, &t) != S_OK)
-      throw 271824;
-    t.OpenCallbackSpec->StartProgressDialog(progressTitle);
+    {
+      NWindows::CThread thread;
+      RINOK(thread.Create(CThreadArchiveOpen::MyThreadFunction, &t));
+      t.OpenCallbackSpec->StartProgressDialog(progressTitle, thread);
+    }
 
     if (t.Result == E_ABORT)
       return t.Result;
 
+    encrypted = t.OpenCallbackSpec->PasswordIsDefined;
     if (t.Result == S_OK)
     {
       // if (openCallbackSpec->PasswordWasAsked)
       {
-        encrypted = t.OpenCallbackSpec->PasswordIsDefined;
         password = t.OpenCallbackSpec->Password;
       }
       *module = library.Detach();
