@@ -3,35 +3,52 @@
 #ifndef __PANEL_H
 #define __PANEL_H
 
-#include "Common/MyCom.h"
+#include "../../../../C/Alloc.h"
 
-#include "Windows/DLL.h"
-#include "Windows/FileFind.h"
-#include "Windows/FileDir.h"
-#include "Windows/Synchronization.h"
-// #include "Windows/Handle.h"
+#include "../../../Common/Defs.h"
+#include "../../../Common/MyCom.h"
 
-// #include "Windows/Control/ToolBar.h"
-// #include "Windows/Control/ReBar.h"
-#include "Windows/Control/ListView.h"
-#include "Windows/Control/Static.h"
-#include "Windows/Control/Edit.h"
-#include "Windows/Control/ComboBox.h"
-#include "Windows/Control/Window2.h"
-#include "Windows/Control/StatusBar.h"
+#include "../../../Windows/DLL.h"
+#include "../../../Windows/FileDir.h"
+#include "../../../Windows/FileFind.h"
+// #include "../../../Windows/Handle.h"
+#include "../../../Windows/Synchronization.h"
+
+#include "../../../Windows/Control/ComboBox.h"
+#include "../../../Windows/Control/Edit.h"
+#include "../../../Windows/Control/ListView.h"
+// #include "../../../Windows/Control/ReBar.h"
+#include "../../../Windows/Control/Static.h"
+#include "../../../Windows/Control/StatusBar.h"
+// #include "../../../Windows/Control/ToolBar.h"
+#include "../../../Windows/Control/Window2.h"
+
+#include "../../Archive/IArchive.h"
+
+#include "ExtractCallback.h"
 
-#include "SysIconUtils.h"
-#include "IFolder.h"
-#include "ViewSettings.h"
 #include "AppState.h"
+#include "IFolder.h"
 // #include "MyCom2.h"
+#include "ProgressDialog2.h"
+#include "SysIconUtils.h"
+
+#ifdef UNDER_CE
+#define NON_CE_VAR(_v_)
+#else
+#define NON_CE_VAR(_v_) _v_
+#endif
 
 const int kParentFolderID = 100;
-const int kPluginMenuStartID = 1000;
-const int kToolbarStartID = 2000;
 
 const int kParentIndex = -1;
 
+#ifdef UNDER_CE
+#define ROOT_FS_FOLDER L"\\"
+#else
+#define ROOT_FS_FOLDER L"C:\\\\"
+#endif
+
 struct CPanelCallback
 {
   virtual void OnTab() = 0;
@@ -54,21 +71,18 @@ struct CItemProperty
   VARTYPE Type;
   int Order;
   bool IsVisible;
+  bool IsRawProp;
   UInt32 Width;
+
+  int Compare(const CItemProperty &a) const { return MyCompare(Order, a.Order); }
 };
 
-inline bool operator<(const CItemProperty &a1, const CItemProperty &a2)
-  { return (a1.Order < a2.Order); }
-
-inline bool operator==(const CItemProperty &a1, const CItemProperty &a2)
-  { return (a1.Order == a2.Order); }
-
 class CItemProperties: public CObjectVector<CItemProperty>
 {
 public:
   int FindItemWithID(PROPID id)
   {
-    for (int i = 0; i < Size(); i++)
+    FOR_VECTOR (i, (*this))
       if ((*this)[i].ID == id)
         return i;
     return -1;
@@ -77,14 +91,26 @@ public:
 
 struct CTempFileInfo
 {
-  UString ItemName;
-  UString FolderPath;
-  UString FilePath;
-  NWindows::NFile::NFind::CFileInfoW FileInfo;
-  void DeleteDirAndFile()
+  UInt32 FileIndex;  // index of file in folder
+  UString RelPath;   // Relative path of file from Folder
+  FString FolderPath;
+  FString FilePath;
+  NWindows::NFile::NFind::CFileInfo FileInfo;
+  bool NeedDelete;
+
+  CTempFileInfo(): FileIndex((UInt32)(Int32)-1), NeedDelete(false) {}
+  void DeleteDirAndFile() const
   {
-    NWindows::NFile::NDirectory::DeleteFileAlways(FilePath);
-    NWindows::NFile::NDirectory::MyRemoveDirectory(FolderPath);
+    if (NeedDelete)
+    {
+      NWindows::NFile::NDir::DeleteFileAlways(FilePath);
+      NWindows::NFile::NDir::RemoveDir(FolderPath);
+    }
+  }
+  bool WasChanged(const NWindows::NFile::NFind::CFileInfo &newFileInfo) const
+  {
+    return newFileInfo.Size != FileInfo.Size ||
+        CompareFileTime(&newFileInfo.MTime, &FileInfo.MTime) != 0;
   }
 };
 
@@ -94,28 +120,39 @@ struct CFolderLink: public CTempFileInfo
   CMyComPtr<IFolderFolder> ParentFolder;
   bool UsePassword;
   UString Password;
+  bool IsVirtual;
 
   UString VirtualPath;
-  CFolderLink(): UsePassword(false) {}
+  CFolderLink(): UsePassword(false), IsVirtual(false) {}
+
+  bool WasChanged(const NWindows::NFile::NFind::CFileInfo &newFileInfo) const
+  {
+    return IsVirtual || CTempFileInfo::WasChanged(newFileInfo);
+  }
+
 };
 
 enum MyMessages
 {
-  kShiftSelectMessage = 20000, // FIXME  = WM_USER + 1,
+  // we can use WM_USER, since we have defined new window class.
+  // so we don't need WM_APP.
+  kShiftSelectMessage = WM_USER + 1,
   kReLoadMessage,
   kSetFocusToListView,
   kOpenItemChanged,
-  kRefreshStatusBar
+  kRefresh_StatusBar
+  #ifdef UNDER_CE
+  , kRefresh_HeaderComboBox
+  #endif
 };
 
 UString GetFolderPath(IFolderFolder * folder);
 
 class CPanel;
 
-class CMyListView: public NWindows::NControl::CListView
+class CMyListView: public NWindows::NControl::CListView2
 {
 public:
-  WNDPROC _origWindowProc;
   CPanel *_panel;
   LRESULT OnMessage(UINT message, WPARAM wParam, LPARAM lParam);
 };
@@ -146,9 +183,44 @@ struct CSelectedState
   CSelectedState(): FocusedItem(-1), SelectFocused(false) {}
 };
 
-class CPanel :public NWindows::NControl::CWindow2
+#ifdef UNDER_CE
+#define MY_NMLISTVIEW_NMITEMACTIVATE NMLISTVIEW
+#else
+#define MY_NMLISTVIEW_NMITEMACTIVATE NMITEMACTIVATE
+#endif
+
+struct CCopyToOptions
 {
-  // FIXME CExtToIconMap _extToIconMap;
+  bool streamMode;
+  bool moveMode;
+  bool testMode;
+  bool includeAltStreams;
+  bool replaceAltStreamChars;
+  bool showErrorMessages;
+
+  UString folder;
+
+  UStringVector hashMethods;
+
+  CVirtFileSystem *VirtFileSystemSpec;
+  ISequentialOutStream *VirtFileSystem;
+  
+  CCopyToOptions():
+      streamMode(false),
+      moveMode(false),
+      testMode(false),
+      includeAltStreams(true),
+      replaceAltStreamChars(false),
+      showErrorMessages(false),
+      VirtFileSystemSpec(NULL),
+      VirtFileSystem(NULL)
+      {}
+};
+  
+
+class CPanel: public NWindows::NControl::CWindow2
+{
+  CExtToIconMap _extToIconMap;
   UINT _baseID;
   int _comboBoxID;
   UINT _statusBarID;
@@ -161,46 +233,58 @@ class CPanel :public NWindows::NControl::CWindow2
   // FIXME virtual bool OnSize(WPARAM wParam, int xSize, int ySize);
   virtual void OnDestroy();
   virtual bool OnNotify(UINT controlID, LPNMHDR lParam, LRESULT &result);
-  void OnComboBoxCommand(UINT code, LPARAM &aParam);
-  // FIXME bool OnNotifyComboBoxEndEdit(PNMCBEENDEDITW info, LRESULT &result);
+
+  void AddComboBoxItem(const UString &name, int iconIndex, int indent, bool addToList);
+
+  bool OnComboBoxCommand(UINT code, LPARAM param, LRESULT &result);
+  
+  #ifndef UNDER_CE
+  
+  LRESULT OnNotifyComboBoxEnter(const UString &s);
+  bool OnNotifyComboBoxEndEdit(PNMCBEENDEDITW info, LRESULT &result);
   #ifndef _UNICODE
   bool OnNotifyComboBoxEndEdit(PNMCBEENDEDIT info, LRESULT &result);
   #endif
-  // FIXME bool OnNotifyReBar(LPNMHDR lParam, LRESULT &result);
-  // FIXME bool OnNotifyComboBox(LPNMHDR lParam, LRESULT &result);
+
+  #endif
+
+  bool OnNotifyReBar(LPNMHDR lParam, LRESULT &result);
+  bool OnNotifyComboBox(LPNMHDR lParam, LRESULT &result);
   void OnItemChanged(NMLISTVIEW *item);
+  void OnNotifyActivateItems();
   bool OnNotifyList(LPNMHDR lParam, LRESULT &result);
-  // FIXME void OnDrag(LPNMLISTVIEW nmListView);
-  // FIXME bool OnKeyDown(LPNMLVKEYDOWN keyDownInfo, LRESULT &result);
-  // FIXME BOOL OnBeginLabelEdit(LV_DISPINFOW * lpnmh);
-  // FIXME BOOL OnEndLabelEdit(LV_DISPINFOW * lpnmh);
+/*
+  void OnDrag(LPNMLISTVIEW nmListView);
+  bool OnKeyDown(LPNMLVKEYDOWN keyDownInfo, LRESULT &result);
+  BOOL OnBeginLabelEdit(LV_DISPINFOW * lpnmh);
+  BOOL OnEndLabelEdit(LV_DISPINFOW * lpnmh);
+*/  
   void OnColumnClick(LPNMLISTVIEW info);
-  // FIXME bool OnCustomDraw(LPNMLVCUSTOMDRAW lplvcd, LRESULT &result);
+  // bool OnCustomDraw(LPNMLVCUSTOMDRAW lplvcd, LRESULT &result);
+
 
 public:
   HWND _mainWindow;
   CPanelCallback *_panelCallback;
 
-/* FIXME BEGIN */
-  HWND GetParent() { return 0; }
-   operator HWND() const { return 0; }
-/* FIXME END */
+  void SysIconsWereChanged() { _extToIconMap.Clear(); }
 
   void DeleteItems(bool toRecycleBin);
-  void DeleteItemsInternal(CRecordVector<UInt32> &indices);
   void CreateFolder();
   void CreateFile();
+  bool CorrectFsPath(const UString &path, UString &result);
+  // bool IsPathForPlugin(const UString &path);
 
 private:
 
   void ChangeWindowSize(int xSize, int ySize);
  
-  void InitColumns();
+  HRESULT InitColumns();
   // void InitColumns2(PROPID sortID);
   void InsertColumn(int index);
 
   void SetFocusedSelectedItem(int index, bool select);
-  void RefreshListCtrl(const UString &focusedName, int focusedPos, bool selectFocused,
+  HRESULT RefreshListCtrl(const UString &focusedName, int focusedPos, bool selectFocused,
       const UStringVector &selectedNames);
 
   void OnShiftSelectMessage();
@@ -210,7 +294,6 @@ private:
   // void OnUpWithShift();
   // void OnDownWithShift();
 public:
-  void myTests();
   void UpdateSelection();
   void SelectSpec(bool selectMode);
   void SelectByType(bool selectMode);
@@ -224,9 +307,16 @@ private:
   // CRecordVector<PROPID> m_ColumnsPropIDs;
 
 public:
-  // FIXME NWindows::NControl::CReBar _headerReBar;
-  // FIXME NWindows::NControl::CToolBar _headerToolBar;
-  NWindows::NControl::CComboBoxEx _headerComboBox;
+  // NWindows::NControl::CReBar _headerReBar;
+  // NWindows::NControl::CToolBar _headerToolBar;
+  NWindows::NControl::
+    #ifdef UNDER_CE
+    CComboBox
+    #else
+    CComboBoxEx
+    #endif
+    _headerComboBox;
+  UStringVector ComboBoxPaths;
   // CMyComboBox _headerComboBox;
   CMyComboBoxEdit _comboBoxEdit;
   CMyListView _listView;
@@ -244,6 +334,10 @@ public:
   CBoolVector _selectedStatusVector;
 
   CSelectedState _selectedState;
+  bool _thereAreDeletedItems;
+  bool _markDeletedItems;
+
+  HWND GetParent();
 
   UInt32 GetRealIndex(const LVITEMW &item) const
   {
@@ -266,9 +360,15 @@ public:
   }
 
   UInt32 _ListViewMode;
-  int _xSize; 
+  int _xSize;
 
   bool _flatMode;
+  bool _flatModeForDisk;
+  bool _flatModeForArc;
+
+  // bool _showNtfsStrems_Mode;
+  // bool _showNtfsStrems_ModeForDisk;
+  // bool _showNtfsStrems_ModeForArc;
 
   bool _dontShowMode;
 
@@ -277,27 +377,59 @@ public:
   
   CObjectVector<CFolderLink> _parentFolders;
   NWindows::NDLL::CLibrary _library;
+  
   CMyComPtr<IFolderFolder> _folder;
+  CMyComPtr<IFolderCompare> _folderCompare;
+  CMyComPtr<IFolderGetItemName> _folderGetItemName;
+  CMyComPtr<IArchiveGetRawProps> _folderRawProps;
+
+  void ReleaseFolder()
+  {
+    _folderCompare.Release();
+    _folderGetItemName.Release();
+    _folderRawProps.Release();
+    _folder.Release();
+    _thereAreDeletedItems = false;
+  }
+
+  void SetNewFolder(IFolderFolder *newFolder)
+  {
+    ReleaseFolder();
+    _folder = newFolder;
+    if (_folder)
+    {
+      _folder.QueryInterface(IID_IFolderCompare, &_folderCompare);
+      _folder.QueryInterface(IID_IFolderGetItemName, &_folderGetItemName);
+      _folder.QueryInterface(IID_IArchiveGetRawProps, &_folderRawProps);
+    }
+  }
+
   // CMyComPtr<IFolderGetSystemIconIndex> _folderGetSystemIconIndex;
 
   UStringVector _fastFolders;
 
   void GetSelectedNames(UStringVector &selectedNames);
   void SaveSelectedState(CSelectedState &s);
-  void RefreshListCtrl(const CSelectedState &s);
-  void RefreshListCtrlSaveFocused();
+  HRESULT RefreshListCtrl(const CSelectedState &s);
+  HRESULT RefreshListCtrlSaveFocused();
+
+  bool GetItem_BoolProp(UInt32 itemIndex, PROPID propID) const;
+  bool IsItem_Deleted(int itemIndex) const;
+  bool IsItem_Folder(int itemIndex) const;
+  bool IsItem_AltStream(int itemIndex) const;
 
   UString GetItemName(int itemIndex) const;
+  void GetItemNameFast(int itemIndex, UString &s) const;
   UString GetItemPrefix(int itemIndex) const;
   UString GetItemRelPath(int itemIndex) const;
-  bool IsItemFolder(int itemIndex) const;
+  UString GetItemFullPath(int itemIndex) const;
   UInt64 GetItemSize(int itemIndex) const;
 
   ////////////////////////
   // PanelFolderChange.cpp
 
   void SetToRootFolder();
-  HRESULT BindToPath(const UString &fullPath, bool &archiveIsOpened, bool &encrypted); // can be prefix 
+  HRESULT BindToPath(const UString &fullPath, const UString &arcFormat, bool &archiveIsOpened, bool &encrypted); // can be prefix
   HRESULT BindToPathAndRefresh(const UString &path);
   void OpenDrivesFolder();
   
@@ -315,6 +447,7 @@ public:
   HRESULT Create(HWND mainWindow, HWND parentWindow,
       UINT id,
       const UString &currentFolderPrefix,
+      const UString &arcFormat,
       CPanelCallback *panelCallback,
       CAppState *appState, bool &archiveIsOpened, bool &encrypted);
   void SetFocusToList();
@@ -334,19 +467,27 @@ public:
       _selectionIsDefined(false),
       _ListViewMode(3),
       _flatMode(false),
+      _flatModeForDisk(false),
+      _flatModeForArc(false),
+
+      // _showNtfsStrems_Mode(false),
+      // _showNtfsStrems_ModeForDisk(false),
+      // _showNtfsStrems_ModeForArc(false),
+
       _xSize(300),
       _mySelectMode(false),
+      _thereAreDeletedItems(false),
+      _markDeletedItems(true),
       _enableItemChangeNotify(true),
       _dontShowMode(false)
   {}
 
-
   void SetExtendedStyle()
   {
-/* FIXME	  
+/* FIXME
     if (_listView != 0)
       _listView.SetExtendedListViewStyle(_exStyle);
-      */
+*/
   }
 
 
@@ -359,37 +500,41 @@ public:
   PROPID _sortID;
   // int _sortIndex;
   bool _ascending;
+  Int32 _isRawSortProp;
+
+  void SetSortRawStatus();
 
   void Release();
   ~CPanel();
-  // FIXME void OnLeftClick(LPNMITEMACTIVATE itemActivate);
-  // FIXME bool OnRightClick(LPNMITEMACTIVATE itemActivate, LRESULT &result);
+  // void OnLeftClick(MY_NMLISTVIEW_NMITEMACTIVATE *itemActivate);
+  // bool OnRightClick(MY_NMLISTVIEW_NMITEMACTIVATE *itemActivate, LRESULT &result);
+  void ShowColumnsContextMenu(int x, int y);
 
   void OnTimer();
   void OnReload();
   bool OnContextMenu(HANDLE windowHandle, int xPos, int yPos);
 
-#ifdef _WIN32 
+#ifdef _WIN32
   CMyComPtr<IContextMenu> _sevenZipContextMenu;
   CMyComPtr<IContextMenu> _systemContextMenu;
   HRESULT CreateShellContextMenu(
       const CRecordVector<UInt32> &operatedIndices,
       CMyComPtr<IContextMenu> &systemContextMenu);
-  void CreateSystemMenu(HMENU menu, 
+  void CreateSystemMenu(HMENU menu,
       const CRecordVector<UInt32> &operatedIndices,
       CMyComPtr<IContextMenu> &systemContextMenu);
-  void CreateSevenZipMenu(HMENU menu, 
+  void CreateSevenZipMenu(HMENU menu,
       const CRecordVector<UInt32> &operatedIndices,
       CMyComPtr<IContextMenu> &sevenZipContextMenu);
-  void CreateFileMenu(HMENU menu, 
+  void CreateFileMenu(HMENU menu,
       CMyComPtr<IContextMenu> &sevenZipContextMenu,
       CMyComPtr<IContextMenu> &systemContextMenu,
       bool programMenu);
   void CreateFileMenu(HMENU menu);
   bool InvokePluginCommand(int id);
-  bool InvokePluginCommand(int id, IContextMenu *sevenZipContextMenu, 
+  bool InvokePluginCommand(int id, IContextMenu *sevenZipContextMenu,
       IContextMenu *systemContextMenu);
-#endif // ifdef _WIN32 
+#endif
 
   void InvokeSystemCommand(const char *command);
   void Properties();
@@ -415,9 +560,14 @@ public:
   void KillSelection();
 
   UString GetFolderTypeID() const;
+  bool IsFolderTypeEqTo(const wchar_t *s) const;
   bool IsRootFolder() const;
   bool IsFSFolder() const;
   bool IsFSDrivesFolder() const;
+  bool IsArcFolder() const;
+  bool IsFsOrDrivesFolder() const { return IsFSFolder() || IsFSDrivesFolder(); }
+  bool IsDeviceDrivesPrefix() const { return _currentFolderPrefix == L"\\\\.\\"; }
+  bool IsFsOrPureDrivesFolder() const { return IsFSFolder() || (IsFSDrivesFolder() && !IsDeviceDrivesPrefix()); }
 
   UString GetFsPath() const;
   UString GetDriveOrNetworkPrefix() const;
@@ -426,53 +576,80 @@ public:
 
   bool _processTimer;
   bool _processNotify;
+  bool _processStatusBar;
 
   class CDisableTimerProcessing
   {
     bool _processTimerMem;
-    bool _processNotifyMem;
 
     CPanel &_panel;
     public:
 
-    CDisableTimerProcessing(CPanel &panel): _panel(panel) 
-    { 
-      Disable();
-    }
+    CDisableTimerProcessing(CPanel &panel): _panel(panel) { Disable(); }
+    ~CDisableTimerProcessing() { Restore(); }
     void Disable()
     {
       _processTimerMem = _panel._processTimer;
-      _processNotifyMem = _panel._processNotify;
-      _panel._processTimer = false; 
-      _panel._processNotify = false; 
+      _panel._processTimer = false;
     }
     void Restore()
     {
-      _panel._processTimer = _processTimerMem; 
-      _panel._processNotify = _processNotifyMem; 
-    }
-    ~CDisableTimerProcessing() 
-    { 
-      Restore();
+      _panel._processTimer = _processTimerMem;
     }
     CDisableTimerProcessing& operator=(const CDisableTimerProcessing &) {; }
   };
 
+  class CDisableNotify
+  {
+    bool _processNotifyMem;
+    bool _processStatusBarMem;
+
+    CPanel &_panel;
+    public:
+
+    CDisableNotify(CPanel &panel): _panel(panel) { Disable(); }
+    ~CDisableNotify() { Restore(); }
+    void Disable()
+    {
+      _processNotifyMem = _panel._processNotify;
+      _processStatusBarMem = _panel._processStatusBar;
+      _panel._processNotify = false;
+      _panel._processStatusBar = false;
+    }
+    void SetMemMode_Enable()
+    {
+      _processNotifyMem  = true;
+      _processStatusBarMem = true;
+    }
+    void Restore()
+    {
+      _panel._processNotify = _processNotifyMem;
+      _panel._processStatusBar = _processStatusBarMem;
+    }
+    CDisableNotify& operator=(const CDisableNotify &) {; }
+  };
+
   // bool _passwordIsDefined;
   // UString _password;
 
-  void RefreshListCtrl();
+  void InvalidateList() { _listView.InvalidateRect(NULL, true); }
+
+  HRESULT RefreshListCtrl();
 
   void MessageBoxInfo(LPCWSTR message, LPCWSTR caption);
   void MessageBox(LPCWSTR message);
+  void MessageBoxWarning(LPCWSTR message);
   void MessageBox(LPCWSTR message, LPCWSTR caption);
   void MessageBoxMyError(LPCWSTR message);
   void MessageBoxError(HRESULT errorCode, LPCWSTR caption);
   void MessageBoxError(HRESULT errorCode);
+  void MessageBoxError2Lines(LPCWSTR message, HRESULT errorCode);
   void MessageBoxLastError(LPCWSTR caption);
   void MessageBoxLastError();
 
-  void MessageBoxErrorLang(UINT resourceID, UInt32 langID);
+  void MessageBoxErrorForUpdate(HRESULT errorCode, UINT resourceID);
+
+  void MessageBoxErrorLang(UINT resourceID);
 
   void OpenFocusedItemAsInternal();
   void OpenSelectedItems(bool internal);
@@ -481,46 +658,55 @@ public:
 
   void OpenFolder(int index);
   HRESULT OpenParentArchiveFolder();
-  HRESULT OpenItemAsArchive(const UString &name,
-      const UString &folderPath,
-      const UString &filePath,
+  HRESULT OpenItemAsArchive(IInStream *inStream,
+      const CTempFileInfo &tempFileInfo,
       const UString &virtualFilePath,
+      const UString &arcFormat,
       bool &encrypted);
-  HRESULT OpenItemAsArchive(const UString &aName);
+  HRESULT OpenItemAsArchive(const UString &relPath, const UString &arcFormat, bool &encrypted);
   HRESULT OpenItemAsArchive(int index);
   void OpenItemInArchive(int index, bool tryInternal, bool tryExternal,
-      bool editMode);
-  HRESULT OnOpenItemChanged(const UString &folderPath, const UString &itemName, bool usePassword, const UString &password);
+      bool editMode, bool useEditor);
+  HRESULT OnOpenItemChanged(UInt32 index, const wchar_t *fullFilePath, bool usePassword, const UString &password);
   LRESULT OnOpenItemChanged(LPARAM lParam);
 
   void OpenItem(int index, bool tryInternal, bool tryExternal);
-  void EditItem();
-  void EditItem(int index);
+  void EditItem(bool useEditor);
+  void EditItem(int index, bool useEditor);
 
   void RenameFile();
   void ChangeComment();
 
   void SetListViewMode(UInt32 index);
-  UInt32 GetListViewMode() const { return _ListViewMode; };
+  UInt32 GetListViewMode() const { return _ListViewMode; }
+  PROPID GetSortID() const { return _sortID; }
 
   void ChangeFlatMode();
-  bool GetFlatMode() const { return _flatMode; };
+  void Change_ShowNtfsStrems_Mode();
+  bool GetFlatMode() const { return _flatMode; }
+  // bool Get_ShowNtfsStrems_Mode() const { return _showNtfsStrems_Mode; }
 
-  void RefreshStatusBar();
-  void OnRefreshStatusBar();
+  bool AutoRefresh_Mode;
+  void Set_AutoRefresh_Mode(bool mode)
+  {
+    AutoRefresh_Mode = mode;
+  }
+  
+  void Post_Refresh_StatusBar();
+  void Refresh_StatusBar();
 
   void AddToArchive();
 
-  void GetFilePaths(const CRecordVector<UInt32> &indices, UStringVector &paths);
+  void GetFilePaths(const CRecordVector<UInt32> &indices, UStringVector &paths, bool allowFolders = false);
   void ExtractArchives();
   void TestArchives();
 
-  HRESULT CopyTo(const CRecordVector<UInt32> &indices, const UString &folder,
-      bool moveMode, bool showErrorMessages, UStringVector *messages,
+  HRESULT CopyTo(CCopyToOptions &options,
+      const CRecordVector<UInt32> &indices,
+      UStringVector *messages,
       bool &usePassword, UString &password);
 
-  HRESULT CopyTo(const CRecordVector<UInt32> &indices, const UString &folder,
-      bool moveMode, bool showErrorMessages, UStringVector *messages)
+  HRESULT CopyTo(CCopyToOptions &options, const CRecordVector<UInt32> &indices, UStringVector *messages)
   {
     bool usePassword = false;
     UString password;
@@ -530,17 +716,17 @@ public:
       usePassword = fl.UsePassword;
       password = fl.Password;
     }
-    return CopyTo(indices, folder, moveMode, showErrorMessages, messages, usePassword, password);
+    return CopyTo(options, indices, messages, usePassword, password);
   }
 
-  HRESULT CopyFrom(const UString &folderPrefix, const UStringVector &filePaths, 
+  HRESULT CopyFrom(bool moveMode, const UString &folderPrefix, const UStringVector &filePaths,
       bool showErrorMessages, UStringVector *messages);
 
   void CopyFromNoAsk(const UStringVector &filePaths);
   void CopyFromAsk(const UStringVector &filePaths);
 
-  // empty folderPath means create new Archive to path of first fileName.
  #ifdef _WIN32
+  // empty folderPath means create new Archive to path of first fileName.
   void DropObject(IDataObject * dataObject, const UString &folderPath);
 
   // empty folderPath means create new Archive to path of first fileName.
@@ -550,7 +736,23 @@ public:
   void RefreshTitle(bool always = false) { _panelCallback->RefreshTitle(always);  }
   void RefreshTitleAlways() { RefreshTitle(true);  }
 
-  UString GetItemsInfoString(const CRecordVector<UInt32> &indices);  
+  UString GetItemsInfoString(const CRecordVector<UInt32> &indices);
+};
+
+class CMyBuffer
+{
+  void *_data;
+public:
+  CMyBuffer(): _data(0) {}
+  operator void *() { return _data; }
+  bool Allocate(size_t size)
+  {
+    if (_data != 0)
+      return false;
+    _data = ::MidAlloc(size);
+    return _data != 0;
+  }
+  ~CMyBuffer() { ::MidFree(_data); }
 };
 
 #endif
