@@ -8,12 +8,14 @@
 #include <kernel/image.h>
 #include <Path.h>
 #else
+#define UINT64 DLL_UINT64 // HP-UX , dlfcn.h defines UINT64 but p7zip also defines UINT64
 #include <dlfcn.h>  // dlopen ...
+#undef UINT64
 #endif
 
 #include "DLL.h"
 #include "Defs.h"
-#ifndef _UNICODE
+#ifdef _UNICODE
 #include "../Common/StringConvert.h"
 #endif
 
@@ -26,14 +28,9 @@
 namespace NWindows {
 namespace NDLL {
 
-CLibrary::~CLibrary()
-{
-  Free();
-}
-
 bool CLibrary::Free()
 {
-TRACEN((printf("CLibrary::Free(%p)\n",(void *)_module)))
+TRACEN((printf("CLibrary::Free(this=%p,%p)\n",(void *)this,(void *)_module)))
   if (_module == 0)
     return true;
 
@@ -57,7 +54,8 @@ static FARPROC local_GetProcAddress(HMODULE module,LPCSTR lpProcName)
   if (module) {
 #ifdef __APPLE_CC__
     char name[MAX_PATHNAME_LEN];
-    sprintf(name,"_%s",lpProcName);
+    snprintf(name,sizeof(name),"_%s",lpProcName);
+    name[sizeof(name)-1] = 0;
     TRACEN((printf("NSLookupSymbolInModule(%p,%s)\n",(void *)module,name)))
     NSSymbol sym;
     sym = NSLookupSymbolInModule((NSModule)module, name);
@@ -72,41 +70,30 @@ static FARPROC local_GetProcAddress(HMODULE module,LPCSTR lpProcName)
 #else
     ptr = dlsym (module, lpProcName);
 #endif
-	TRACEN((printf("CLibrary::GetProcAddress : dlsym(%p,%s)=%p\n",(void *)module,lpProcName,ptr)))
+	TRACEN((printf("CLibrary::GetProc : dlsym(%p,%s)=%p\n",(void *)module,lpProcName,ptr)))
   }
   return (FARPROC)ptr;
 }
 
-FARPROC CLibrary::GetProcAddress(LPCSTR lpProcName) const
+FARPROC CLibrary::GetProc(LPCSTR lpProcName) const
 {
-  TRACEN((printf("CLibrary::GetProcAddress(%p,%s)\n",(void *)_module,lpProcName)))
+  TRACEN((printf("CLibrary::GetProc(%p,%s)\n",(void *)_module,lpProcName)))
   return local_GetProcAddress(_module,lpProcName);
 }
 
-bool CLibrary::LoadOperations(HMODULE newModule)
-{
-  if (newModule == NULL)
-    return false;
-  if(!Free())
-    return false;
-  _module = newModule;
-  return true;
-}
-
-// bool CLibrary::LoadEx(LPCTSTR fileName, DWORD flags)
-// {
-//   /* return LoadOperations(::LoadLibraryEx(fileName, NULL, flags)); */
-//   return this->Load(fileName);
-// }
-
-typedef BOOL (*t_DllMain)(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved);
-
 bool CLibrary::Load(LPCTSTR lpLibFileName)
 {
-  // HMODULE handler = ::LoadLibrary(fileName);
+  if(!Free())
+    return false;
+
   void *handler = 0;
   char  name[MAX_PATHNAME_LEN+1];
+#ifdef _UNICODE
+  AString name2 = UnicodeStringToMultiByte(lpLibFileName);
+  strcpy(name,nameWindowToUnix((const char *)name2));
+#else
   strcpy(name,nameWindowToUnix(lpLibFileName));
+#endif
   
   // replace ".dll" with ".so"
   size_t len = strlen(name);
@@ -114,6 +101,8 @@ bool CLibrary::Load(LPCTSTR lpLibFileName)
     strcpy(name+len-4,".so");
   }
 
+  TRACEN((printf("CLibrary::Load(this=%p,%ls) => %s\n",(void *)this,lpLibFileName,name)))
+
 #ifdef __APPLE_CC__
   NSObjectFileImage image;
   NSObjectFileImageReturnCode nsret;
@@ -121,7 +110,7 @@ bool CLibrary::Load(LPCTSTR lpLibFileName)
   nsret = NSCreateObjectFileImageFromFile (name, &image);
   if (nsret == NSObjectFileImageSuccess) {
      TRACEN((printf("NSCreateObjectFileImageFromFile(%s) : OK\n",name)))
-     handler = (HMODULE)NSLinkModule(image,lpLibFileName,NSLINKMODULE_OPTION_RETURN_ON_ERROR
+     handler = (HMODULE)NSLinkModule(image,name,NSLINKMODULE_OPTION_RETURN_ON_ERROR
            | NSLINKMODULE_OPTION_PRIVATE | NSLINKMODULE_OPTION_BINDNOW);
   } else {
      TRACEN((printf("NSCreateObjectFileImageFromFile(%s) : ERROR\n",name)))
@@ -130,10 +119,14 @@ bool CLibrary::Load(LPCTSTR lpLibFileName)
   // normalize path (remove things like "./", "..", etc..), otherwise it won't work
   BPath p(name, NULL, true);
   status_t err = B_OK;
-  handler = (HMODULE)load_add_on(p.Path());
-  if (handler < 0) {
+  image_id image = load_add_on(p.Path());
+TRACEN((printf("load_add_on(%s)=%d\n",p.Path(),(int)image)))
+  if (image < 0) {
     err = (image_id)handler;
     handler = 0;
+  } else {
+    err = 0;
+    handler = (HMODULE)image;
   }
 #else
   int options_dlopen = 0;
@@ -144,23 +137,25 @@ bool CLibrary::Load(LPCTSTR lpLibFileName)
   options_dlopen |= RTLD_NOW;
 #endif
 #ifdef RTLD_GROUP
-  options_dlopen |= RTLD_GROUP; // for solaris
+  #if ! (defined(hpux) || defined(__hpux))
+  options_dlopen |= RTLD_GROUP; // mainly for solaris but not for HPUX
+  #endif
 #endif
+  TRACEN((printf("CLibrary::Load - dlopen(%s,0x%d)\n",name,options_dlopen)))
   handler = dlopen(name,options_dlopen);
 #endif // __APPLE_CC__
-  TRACEN((printf("LoadLibraryA(%s)=%p\n",name,handler)))
+  TRACEN((printf("CLibrary::Load(%s) => %p\n",name,handler)))
   if (handler) {
 
-    // Call DllMain() like in Windows
-    t_DllMain p_DllMain = (t_DllMain)local_GetProcAddress (handler, "DllMain");
-    if (p_DllMain)
-    {
-      p_DllMain(0,DLL_PROCESS_ATTACH,0);
-    }
+    // Call DllMain() like in Windows : useless now
+
     // Propagate the value of global_use_utf16_conversion into the plugins
     int *tmp = (int *)local_GetProcAddress(handler,"global_use_utf16_conversion");
     if (tmp) *tmp = global_use_utf16_conversion;
 
+    tmp = (int *)local_GetProcAddress(handler,"global_use_lstat");
+    if (tmp) *tmp = global_use_lstat;
+
     // test construtors calls
     void (*fctTest)(void) = (void (*)(void))local_GetProcAddress(handler,"sync_TestConstructor");
     if (fctTest) fctTest();
@@ -171,29 +166,36 @@ bool CLibrary::Load(LPCTSTR lpLibFileName)
     int num_err;
     const char *file,*err;
     NSLinkEditError(&c,&num_err,&file,&err);
-    printf("Can't load '%s' (%s)\n", lpLibFileName,err);
+    printf("Can't load '%ls' (%s)\n", lpLibFileName,err);
 #elif ENV_BEOS
-    printf("Can't load '%s' (%s)\n", lpLibFileName,strerror(err));
+    printf("Can't load '%ls' (%s)\n", lpLibFileName,strerror(err));
 #else
-    printf("Can't load '%s' (%s)\n", lpLibFileName,dlerror());
+    printf("Can't load '%ls' (%s)\n", lpLibFileName,dlerror());
 #endif
   } 
 
-  return LoadOperations(handler);
+  _module = handler;
+  TRACEN((printf("CLibrary::Load(this=%p,%ls) => _module=%p\n",(void *)this,lpLibFileName,_module)))
+
+  return true;
 }
 
-#ifndef _UNICODE
-// bool CLibrary::LoadEx(LPCWSTR fileName, DWORD flags)
-// {
-//   return LoadEx(UnicodeStringToMultiByte(fileName, CP_ACP), flags);
-// }
+#ifndef _SFX
 
-
-bool CLibrary::Load(LPCWSTR fileName)
+FString GetModuleDirPrefix()
 {
-  return Load(UnicodeStringToMultiByte(fileName, CP_ACP));
+  FString s;
+
+  const char *p7zip_home_dir = getenv("P7ZIP_HOME_DIR");
+  if (p7zip_home_dir) {
+    return MultiByteToUnicodeString(p7zip_home_dir,CP_ACP);
+  }
+
+  return FTEXT(".") FSTRING_PATH_SEPARATOR;
 }
+
 #endif
 
+
 }}
 
