@@ -18,7 +18,7 @@
 #include <sys/types.h>
 #include <utime.h>
 
-#ifdef HAVE_LSTAT
+#ifdef ENV_HAVE_LSTAT
 #define FD_LINK (-2)
 #endif
 
@@ -72,7 +72,7 @@ bool CFileBase::Create(LPCSTR filename, DWORD dwDesiredAccess,
   // printf("##DBG open(%s,0x%x,%o)##\n",name,flags,(unsigned)mode);
 
   _fd = -1;
-#ifdef HAVE_LSTAT
+#ifdef ENV_HAVE_LSTAT
    if ((global_use_lstat) && (ignoreSymbolicLink == false))
    {
      _size = readlink(name, _buffer, sizeof(_buffer)-1);
@@ -98,7 +98,7 @@ bool CFileBase::Create(LPCSTR filename, DWORD dwDesiredAccess,
     UString ustr = MultiByteToUnicodeString(AString(name), 0);
     AString resultString;
     int is_good = 1;
-    for (int i = 0; i < ustr.Length(); i++)
+    for (int i = 0; i < ustr.Len(); i++)
     {
       if (ustr[i] >= 256) {
         is_good = 0;
@@ -113,7 +113,7 @@ bool CFileBase::Create(LPCSTR filename, DWORD dwDesiredAccess,
   }
 
   if (_fd == -1) {
-    /* !HAVE_LSTAT : an invalid symbolic link => errno == ENOENT */
+    /* !ENV_HAVE_LSTAT : an invalid symbolic link => errno == ENOENT */
     return false;
   } else {
     _unix_filename = name;
@@ -142,7 +142,7 @@ bool CFileBase::Close()
   if(_fd == -1)
     return true;
 
-#ifdef HAVE_LSTAT
+#ifdef ENV_HAVE_LSTAT
   if(_fd == FD_LINK) {
     _fd = -1;
     return true;
@@ -180,7 +180,7 @@ bool CFileBase::GetLength(UINT64 &length) const
      return false;
   }
 
-#ifdef HAVE_LSTAT  
+#ifdef ENV_HAVE_LSTAT  
   if (_fd == FD_LINK) {
     length = _size;
     return true;
@@ -212,7 +212,7 @@ bool CFileBase::Seek(INT64 distanceToMove, DWORD moveMethod, UINT64 &newPosition
      return false;
   }
 
-#ifdef HAVE_LSTAT
+#ifdef ENV_HAVE_LSTAT
   if (_fd == FD_LINK) {
     INT64 offset;
     switch (moveMethod) {
@@ -308,7 +308,7 @@ bool CInFile::Read(void *buffer, UINT32 bytesToRead, UINT32 &bytesRead)
     return TRUE;
   }
 
-#ifdef HAVE_LSTAT
+#ifdef ENV_HAVE_LSTAT
   if (_fd == FD_LINK) {
     if (_offset >= _size) {
       bytesRead = 0;
@@ -360,6 +360,11 @@ bool COutFile::Create(LPCTSTR fileName, bool createAlways)
   return Open(fileName, GetCreationDisposition(createAlways));
 }
 
+bool COutFile::CreateAlways(LPCTSTR fileName, DWORD /* flagsAndAttributes */ )
+{
+  return Open(fileName, true); // FIXME
+}
+
 #ifndef _UNICODE
 
 bool COutFile::Open(LPCWSTR fileName, DWORD shareMode, 
