@@ -16,10 +16,10 @@
 #undef UInt32
 #endif
 
-#ifdef HAVE_WCHAR__H
+#ifdef ENV_HAVE_WCHAR__H
 #include <wchar.h>
 #endif
-#ifdef HAVE_LOCALE
+#ifdef ENV_HAVE_LOCALE
 #include <locale.h>
 #endif
 
@@ -37,15 +37,56 @@
 #include "Common/StringConvert.cpp"
 #include "Common/StdOutStream.cpp"
 #include "Common/IntToString.cpp"
+#include "Common/UTFConvert.cpp"
 
 #include "Windows/Synchronization.cpp"
 #include "Windows/FileFind.cpp"
-#include "Windows/Time.cpp"
+#include "Windows/TimeUtils.cpp"
 #include "../C/Threads.c"
+#include "../../C/Ppmd.h"
+
+int g_CodePage = -1;
+
+/* FIXME */
+
+LPSTR WINAPI CharNextA( LPCSTR ptr ) {
+  if (!*ptr)
+    return (LPSTR)ptr;
+  return (LPSTR)(ptr + 1); // p7zip search only for ASCII characters like '/' so no need to worry about current locale
+}
+
+
+int MyStringCompare(const char *s1, const char *s2)
+{
+  while (true)
+  {
+    unsigned char c1 = (unsigned char)*s1++;
+    unsigned char c2 = (unsigned char)*s2++;
+    if (c1 < c2) return -1;
+    if (c1 > c2) return 1;
+    if (c1 == 0) return 0;
+  }
+}
+
+int MyStringCompare(const wchar_t *s1, const wchar_t *s2)
+{
+  while (true)
+  {
+    wchar_t c1 = *s1++;
+    wchar_t c2 = *s2++;
+    if (c1 < c2) return -1;
+    if (c1 > c2) return 1;
+    if (c1 == 0) return 0;
+  }
+}
+
+
+
+/* FIXME */
 
 using namespace NWindows;
 
-#if  defined(HAVE_WCHAR__H) && defined(HAVE_MBSTOWCS) && defined(HAVE_WCSTOMBS)
+#if  defined(ENV_HAVE_WCHAR__H) && defined(ENV_HAVE_MBSTOWCS) && defined(ENV_HAVE_WCSTOMBS)
 void test_mbs(void) {
   wchar_t wstr1[256] = {
                          L'e',
@@ -88,19 +129,19 @@ void test_mbs(void) {
   }
 
   UString ustr(wstr1);
-  assert(ustr.Length() == (int)len1);
+  assert(ustr.Len() == (int)len1);
 
   AString  ansistr(astr);
-  assert(ansistr.Length() == (int)len0);
+  assert(ansistr.Len() == (int)len0);
 
   ansistr = UnicodeStringToMultiByte(ustr);
-  assert(ansistr.Length() == (int)len0);
+  assert(ansistr.Len() == (int)len0);
 
   assert(strcmp(ansistr,astr) == 0);
   assert(wcscmp(ustr,wstr1) == 0);
 
   UString ustr2 = MultiByteToUnicodeString(astr);
-  assert(ustr2.Length() == (int)len1);
+  assert(ustr2.Len() == (int)len1);
   assert(wcscmp(ustr2,wstr1) == 0);
 }
 #endif
@@ -225,12 +266,21 @@ static void test_time()
 
 static void test_time2()
 {
-	printf("Test Time (2):\n");
+        UInt32 dosTime = 0x30d0094C;
+        FILETIME utcFileTime;
+        FILETIME localFileTime;
+        FILETIME localFileTime2;
+        UInt32 dosTime2 = 0;
+
+        printf("Test Time (2):\n");
+        printf("===========\n");
+        NTime::DosTimeToFileTime(dosTime, localFileTime);
+        NTime::FileTimeToDosTime(localFileTime, dosTime2);
+        assert(dosTime == dosTime2);
+
+        printf("Test Time (3):\n");
 	printf("===========\n");
 	/* DosTime To utcFileTime */
-	UInt32 dosTime = 0x30d0094C;
-	FILETIME utcFileTime;
-        FILETIME localFileTime;
 
 	if (NTime::DosTimeToFileTime(dosTime, localFileTime)) /* DosDateTimeToFileTime */
 	{
@@ -244,8 +294,7 @@ static void test_time2()
 
 
 	/* utcFileTime to DosTime */
-        FILETIME localFileTime2 = { 0, 0 };
-	UInt32 dosTime2 = 0;
+
         FileTimeToLocalFileTime(&utcFileTime, &localFileTime2);
         NTime::FileTimeToDosTime(localFileTime2, dosTime2);  /* FileTimeToDosDateTime */
 
@@ -299,7 +348,7 @@ static void test_semaphore()
 
 static int threads_count = 0;
 
-static THREAD_FUNC_RET_TYPE thread_fct(void *param) {
+static THREAD_FUNC_RET_TYPE thread_fct(void *  /* param */ ) {
 	threads_count++;
 	return 0;
 }
@@ -404,12 +453,12 @@ void  testMaxOSX_stringConvert()
      // dumpWStr("1",&ustr[0]);
 
      assert(MyStringCompare(&ustr[0],tab[i].ustr) == 0);
-     assert(ustr.Length() == wcslen(tab[i].ustr) );
+     assert(ustr.Len() == wcslen(tab[i].ustr) );
 
 
      AString astr = GetAnsiString(ustr);
      assert(MyStringCompare(&astr[0],tab[i].astr) == 0);
-     assert(astr.Length() == strlen(tab[i].astr) );
+     assert(astr.Len() == strlen(tab[i].astr) );
 
      i++;
    }
@@ -585,11 +634,11 @@ static void  test_AString()
 
    a = "abc";
    assert(MyStringCompare(&a[0],"abc") == 0);
-   assert(a.Length() == 3);
+   assert(a.Len() == 3);
 
    a = GetAnsiString(L"abc");
    assert(MyStringCompare(&a[0],"abc") == 0);
-   assert(a.Length() == 3);
+   assert(a.Len() == 3);
 }
 
 
@@ -625,11 +674,11 @@ static void test_UString()
 
    u1 = L"abc";
    assert(MyStringCompare(&u1[0],L"abc") == 0);
-   assert(u1.Length() == 3);
+   assert(u1.Len() == 3);
 
    u1 = GetUnicodeString("abc");
    assert(MyStringCompare(&u1[0],L"abc") == 0);
-   assert(u1.Length() == 3);
+   assert(u1.Len() == 3);
 }
 
 /****************************************************************************************/
@@ -638,7 +687,7 @@ int main() {
   // return test_thread();
 
 
-#ifdef HAVE_LOCALE
+#ifdef ENV_HAVE_LOCALE
   setlocale(LC_ALL,"");
 #endif
 
@@ -663,6 +712,20 @@ int main() {
 #ifdef __APPLE_CC__
   printf("sizeof(UniChar) : %d\n",(int)sizeof(UniChar));
 #endif
+  printf("sizeof(CPpmd_See) : %d\n",(int)sizeof(CPpmd_See));
+  printf("sizeof(CPpmd_State) : %d\n",(int)sizeof(CPpmd_State));
+
+  // size tests
+  assert(sizeof(Byte)==1);
+  assert(sizeof(UInt16)==2);
+  assert(sizeof(UInt32)==4);
+  assert(sizeof(UINT32)==4);
+  assert(sizeof(UInt64)==8);
+  assert(sizeof(UINT64)==8);
+
+  // alignement tests
+  assert(sizeof(CPpmd_See)==4);
+  assert(sizeof(CPpmd_State)==6);
 
   union {
 	Byte b[2];
@@ -678,7 +741,7 @@ int main() {
     printf("CPU : unknown endianess\n");
   }
 
-#if  defined(HAVE_WCHAR__H) && defined(HAVE_MBSTOWCS) && defined(HAVE_WCSTOMBS)
+#if  defined(ENV_HAVE_WCHAR__H) && defined(ENV_HAVE_MBSTOWCS) && defined(ENV_HAVE_WCSTOMBS)
   test_mbs();
 #endif
 
